Package: greeks
Title: Sensitivities of Prices of Financial Options and Implied
        Volatilities
Version: 1.5.6
Authors@R: 
    person(given = "Anselm",
           family = "Hudde",
           role = c("aut", "cre"), 
           email = "anselmhudde@gmx.de",
           comment = c(ORCID = "0000-0002-5652-2815"))
Description: Methods to calculate sensitivities of financial option prices for
 European, geometric and arithmetic Asian, and American options, with various
 payoff functions in the Black Scholes model, and in more general jump diffusion
 models. A shiny app to interactively plot the results is included. Furthermore,
 methods to compute implied volatilities are provided for a wide range of option
 types and  custom payoff functions. Classical formulas are implemented for
 European options in the Black Scholes Model, as is presented in Hull, J. C.
 (2017), Options, Futures, and Other Derivatives.
 In the case of Asian options, Malliavin Monte Carlo Greeks are implemented, see
 Hudde, A. & Rüschendorf, L. (2023). European and Asian Greeks for exponential
 Lévy processes. <doi:10.1007/s11009-023-10014-5>. For American
 options, the Binomial Tree  Method is implemented, as is presented in Hull,
 J. C. (2017). 
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), R.rsp
Config/testthat/edition: 3
Imports: magrittr, dqrng, Rcpp, tibble, ggplot2, plotly, shiny, tidyr
LinkingTo: Rcpp
URL: https://github.com/ahudde/greeks
BugReports: https://github.com/ahudde/greeks/issues
NeedsCompilation: yes
Packaged: 2026-07-17 13:19:40 UTC; ahudde
Author: Anselm Hudde [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5652-2815>)
Maintainer: Anselm Hudde <anselmhudde@gmx.de>
VignetteBuilder: knitr
Repository: CRAN
Date/Publication: 2026-07-17 13:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-17 16:53:52 UTC; unix
Archs: greeks.so.dSYM

# Generated by roxygen2: do not edit by hand

export("%>%")
export(BS_European_Greeks)
export(BS_Geometric_Asian_Greeks)
export(BS_Implied_Volatility)
export(BS_Malliavin_Asian_Greeks)
export(Binomial_American_Greeks)
export(Greeks)
export(Greeks_UI)
export(Implied_Volatility)
export(Malliavin_Asian_Greeks)
export(Malliavin_European_Greeks)
export(Malliavin_Geometric_Asian_Greeks)
import("Rcpp")
import("ggplot2")
import("shiny")
import("stats")
import("tibble")
import("tidyr")
importFrom("dqrng","dqrnorm")
importFrom("dqrng","dqset.seed")
importFrom("plotly","ggplotly")
importFrom("plotly","plotlyOutput")
importFrom("plotly","renderPlotly")
importFrom(magrittr,"%>%")
useDynLib(greeks, .registration=TRUE)

# greeks
derivatives

[![Build Status](https://travis-ci.org/timkpaine/greeks.svg?branch=master)](https://travis-ci.org/timkpaine/greeks)
[![GitHub issues](https://img.shields.io/github/issues/timkpaine/greeks.svg)]()
[![codecov](https://codecov.io/gh/timkpaine/greeks/branch/master/graph/badge.svg)](https://codecov.io/gh/timkpaine/greeks)
[![BCH compliance](https://bettercodehub.com/edge/badge/timkpaine/greeks?branch=master)](https://bettercodehub.com/)
[![PyPI](https://img.shields.io/pypi/l/greeks.svg)](https://pypi.python.org/pypi/greeks)
[![PyPI](https://img.shields.io/pypi/v/greeks.svg)](https://pypi.python.org/pypi/greeks)
[![Docs](https://img.shields.io/readthedocs/greeks.svg)](https://greeks.readthedocs.io)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(greeks)

## -----------------------------------------------------------------------------
american_put <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = c("fair_value", "delta", "gamma", "vega", "theta", "rho"),
  steps = 200
)

round(american_put, 4)

## -----------------------------------------------------------------------------
round(
  Greeks(
    initial_price = 100,
    exercise_price = 100,
    r = 0.03,
    time_to_maturity = 1,
    dividend_yield = 0,
    volatility = 0.3,
    payoff = "put",
    option_type = "American",
    greek = c("fair_value", "delta", "rho"),
    steps = 200
  ),
  4
)

## -----------------------------------------------------------------------------
european_put_value <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "fair_value"
)

american_put_value <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "fair_value",
  steps = 400
)

round(
  c(
    european_put = european_put_value,
    american_put = american_put_value,
    early_exercise_premium = american_put_value - european_put_value
  ),
  4
)

## -----------------------------------------------------------------------------
steps <- c(25, 50, 100, 200, 400)

prices <- vapply(
  steps,
  function(n_steps) {
    Binomial_American_Greeks(
      initial_price = 100,
      exercise_price = 100,
      r = 0.03,
      time_to_maturity = 1,
      dividend_yield = 0,
      volatility = 0.3,
      payoff = "put",
      greek = "fair_value",
      steps = n_steps
    )
  },
  numeric(1)
)

data.frame(steps = steps, fair_value = round(prices, 4))

## -----------------------------------------------------------------------------
fair_value_at <- function(initial_price) {
  Binomial_American_Greeks(
    initial_price = initial_price,
    exercise_price = 100,
    r = 0.03,
    time_to_maturity = 1,
    dividend_yield = 0,
    volatility = 0.3,
    payoff = "put",
    greek = "fair_value",
    steps = 200
  )
}

step_size <- 0.01
finite_difference_delta <-
  (fair_value_at(100 + step_size) - fair_value_at(100 - step_size)) /
  (2 * step_size)

reported_delta <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "delta",
  steps = 200,
  eps = step_size
)

round(
  c(
    reported_delta = reported_delta,
    finite_difference_delta = finite_difference_delta,
    absolute_error = abs(reported_delta - finite_difference_delta)
  ),
  8
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(greeks)

## -----------------------------------------------------------------------------
european_put <- BS_European_Greeks(
  initial_price = 120,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 4.5,
  dividend_yield = 0.015,
  volatility = 0.22,
  payoff = "put",
  greek = c("fair_value", "delta", "gamma", "vega", "theta", "rho")
)

round(european_put, 4)

## -----------------------------------------------------------------------------
round(
  Greeks(
    initial_price = 120,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 4.5,
    dividend_yield = 0.015,
    volatility = 0.22,
    payoff = "put",
    greek = c("fair_value", "delta", "gamma")
  ),
  4
)

## -----------------------------------------------------------------------------
digital_call <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "cash_or_nothing_call",
  greek = c("fair_value", "delta", "vega")
)

round(digital_call, 4)

## -----------------------------------------------------------------------------
base_args <- list(
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "call"
)

fair_value_at <- function(initial_price) {
  do.call(
    BS_European_Greeks,
    c(base_args, list(initial_price = initial_price, greek = "fair_value"))
  )
}

step_size <- 1e-4
finite_difference_delta <-
  (fair_value_at(100 + step_size) - fair_value_at(100 - step_size)) /
  (2 * step_size)

exact_delta <- do.call(
  BS_European_Greeks,
  c(base_args, list(initial_price = 100, greek = "delta"))
)

round(
  c(
    exact_delta = exact_delta,
    finite_difference_delta = finite_difference_delta,
    absolute_error = abs(exact_delta - finite_difference_delta)
  ),
  8
)

## -----------------------------------------------------------------------------
greeks_to_compare <- c("fair_value", "delta", "vega", "theta", "rho", "gamma")

exact <- BS_European_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare
)

monte_carlo <- Malliavin_European_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare,
  paths = 50000,
  seed = 42,
  antithetic = TRUE
)

round(rbind(exact = exact, malliavin_monte_carlo = monte_carlo), 4)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(greeks)

## -----------------------------------------------------------------------------
true_volatility <- 0.28

option_price <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  volatility = true_volatility,
  payoff = "call",
  greek = "fair_value"
)

implied_volatility <- BS_Implied_Volatility(
  option_price = option_price,
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  payoff = "call",
  start_volatility = 0.2
)

round(
  c(
    option_price = option_price,
    true_volatility = true_volatility,
    implied_volatility = implied_volatility
  ),
  6
)

## -----------------------------------------------------------------------------
reconstructed_price <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  volatility = implied_volatility,
  payoff = "call",
  greek = "fair_value"
)

round(
  c(
    original_price = option_price,
    reconstructed_price = reconstructed_price,
    absolute_error = abs(option_price - reconstructed_price)
  ),
  10
)

## -----------------------------------------------------------------------------
geometric_true_volatility <- 0.35

geometric_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = geometric_true_volatility,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

geometric_implied_volatility <- Implied_Volatility(
  option_price = geometric_price,
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  option_type = "Geometric Asian",
  payoff = "put"
)

geometric_reconstructed_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = geometric_implied_volatility,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

round(
  c(
    original_price = geometric_price,
    true_volatility = geometric_true_volatility,
    implied_volatility = geometric_implied_volatility,
    reconstructed_price = geometric_reconstructed_price,
    absolute_error = abs(geometric_price - geometric_reconstructed_price)
  ),
  6
)

## -----------------------------------------------------------------------------
near_zero_volatility_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 1e-12,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

round(near_zero_volatility_price, 6)

#' @title
#' Computes the implied volatility for European put- and call options in the
#' Black Scholes model via Halley's method.
#'
#' @description For the definition of *implied volatility* see
#' [Implied_Volatility].
#' [BS_Implied_Volatility] offers a very fast implementation for European put-
#' and call options applying Halley's method (see
#'
#' [en.wikipedia.org/wiki/Halley%27s_method](https://en.wikipedia.org/wiki/Halley%27s_method)).
#'
#' @export
#'
#' @seealso [Implied_Volatility] for American and Asian options, and for
#' digital payoff functions
#'
#' @import "stats"
#' @import "Rcpp"
#'
#' @param option_price - current price of the option
#' @param initial_price - initial price of the underlying asset.
#' @param exercise_price - strike price of the option.
#' @param r - risk-free interest rate.
#' @param time_to_maturity - time to maturity.
#' @param dividend_yield - dividend yield.
#' @param payoff - the payoff function, a string in ("call", "put").
#' @param start_volatility - the volatility value to start the approximation
#' @param precision - precision of the result
#'
#' @useDynLib greeks, .registration=TRUE
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples BS_Implied_Volatility(option_price = 27, initial_price = 100,
#' exercise_price = 100, r = 0.03, time_to_maturity = 5, dividend_yield = 0.015,
#' payoff = "call")

BS_Implied_Volatility <-
  function(option_price,
           initial_price = 100,
           exercise_price = 100,
           r = 0,
           time_to_maturity = 1,
           dividend_yield = 0,
           payoff = "call",
           start_volatility = 0.3,
           precision = 1e-9) {

    ## check if option price can be obtained

    option_price_zero_vol <-
      BS_European_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = 1e-12,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = "fair_value"
      )

    if (option_price <= option_price_zero_vol) {
      stop("Option price is too low. Implied volatility is not defined.")
    }

    ## Start computation

    volatility <- start_volatility

    d1 <- (log(initial_price/exercise_price) +
             (r - dividend_yield + (volatility^2)/2) * time_to_maturity) /
      (volatility * sqrt(time_to_maturity))

    d2 <- d1 - volatility * sqrt(time_to_maturity)

    if (payoff == "call") {

      fair_value <-
        initial_price * exp(-dividend_yield*time_to_maturity) * pnorm(d1) -
        exp(-r*time_to_maturity) * exercise_price * pnorm(d2)

    } else if (payoff == "put") {

      fair_value <-
        exp(-r*time_to_maturity) * exercise_price * pnorm(-d2) -
        initial_price * exp(-dividend_yield * time_to_maturity) * pnorm(-d1)

    }

    while (TRUE) {

      vega <-
        initial_price*exp(-dividend_yield*time_to_maturity) * dnorm(d1) *
        sqrt(time_to_maturity)

      vomma <-
        initial_price * exp(-dividend_yield * time_to_maturity) *
        dnorm(d1) * sqrt(time_to_maturity) * d1 * d2 / volatility

      volatility <-
        volatility -
        (2 * (fair_value - option_price) * vega) / (2 * vega^2 - (fair_value - option_price) * vomma)

      if (abs(fair_value - option_price) < precision) {
        return(volatility)
      }

      d1 <- (log(initial_price/exercise_price) +
               (r - dividend_yield + (volatility^2)/2) * time_to_maturity) /
        (volatility * sqrt(time_to_maturity))

      d2 <- d1 - volatility * sqrt(time_to_maturity)

      if (payoff == "call") {

        fair_value <-
          initial_price * exp(-dividend_yield*time_to_maturity) * pnorm(d1) -
          exp(-r*time_to_maturity) * exercise_price * pnorm(d2)

      } else if (payoff == "put") {

        fair_value <-
          exp(-r*time_to_maturity) * exercise_price * pnorm(-d2) -
          initial_price * exp(-dividend_yield * time_to_maturity) * pnorm(-d1)

      }

    }

  }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(greeks)

## -----------------------------------------------------------------------------
geometric_exact <- BS_Geometric_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma")
)

round(geometric_exact, 4)

## -----------------------------------------------------------------------------
geometric_price_at <- function(volatility) {
  BS_Geometric_Asian_Greeks(
    initial_price = 110,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1.5,
    dividend_yield = 0.01,
    volatility = volatility,
    payoff = "call",
    greek = "fair_value"
  )
}

step_size <- 1e-4
finite_difference_vega <-
  (geometric_price_at(0.25 + step_size) -
     geometric_price_at(0.25 - step_size)) /
  (2 * step_size)

round(
  c(
    exact_vega = geometric_exact["vega"],
    finite_difference_vega = finite_difference_vega,
    absolute_error = abs(geometric_exact["vega"] - finite_difference_vega)
  ),
  8
)

## -----------------------------------------------------------------------------
greeks_to_compare <- c("fair_value", "delta", "rho", "vega")

geometric_monte_carlo <- Malliavin_Geometric_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare,
  paths = 20000,
  steps = 24,
  seed = 42,
  antithetic = TRUE
)

round(
  rbind(
    exact = geometric_exact[greeks_to_compare],
    malliavin_monte_carlo = geometric_monte_carlo
  ),
  4
)

## -----------------------------------------------------------------------------
arithmetic_asian <- BS_Malliavin_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega"),
  paths = 10000,
  steps = 24,
  seed = 42
)

round(arithmetic_asian, 4)

## -----------------------------------------------------------------------------
round(
  Greeks(
    initial_price = 110,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1.5,
    dividend_yield = 0.01,
    volatility = 0.25,
    payoff = "call",
    option_type = "Asian",
    greek = c("fair_value", "delta", "rho", "vega")
  ),
  4
)

## -----------------------------------------------------------------------------
price_grid <- Malliavin_Geometric_Asian_Greeks(
  initial_price = c(95, 100, 105),
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "put",
  greek = c("fair_value", "delta"),
  paths = 5000,
  steps = 12,
  seed = 42
)

round(price_grid, 4)

## -----------------------------------------------------------------------------
digital_call <- function(x, exercise_price) {
  ifelse(x >= exercise_price, 1, 0)
}

invisible(capture.output(
  custom_digital <- Malliavin_Geometric_Asian_Greeks(
    initial_price = 100,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1,
    volatility = 0.25,
    payoff = digital_call,
    greek = c("fair_value", "delta"),
    paths = 5000,
    steps = 12,
    seed = 42
  )
))

round(custom_digital, 4)

#' @title
#' Computes the Greeks of a European option with the Malliavin Monte Carlo
#' Method in the Black Scholes model
#'
#' @description
#' For details on the definition of Greeks see [Greeks].
#' For a description of Malliavin Monte Carlo Methods for Greeks see for example
#' (Hudde & Rüschendorf, 2023).
#'
#' @export
#'
#' @seealso [BS_European_Greeks] for the exact and fast implementation for
#' call-, put- and digital payoff functions
#'
#' @import "stats"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset
#' @param exercise_price - strike price of the option
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param volatility - volatility of the underlying asset
#' @param payoff - the payoff function, either a string in ("call", "put",
#' "cash_or_nothing_call", "cash_or_nothing_call", "asset_or_nothing_call",
#' "asset_or_nothing_put"), or a function
#' @param greek - Greeks to be calculated in ("fair_value", "delta", "vega",
#' "theta", "rho", "gamma")
#' @param model - the model to be chosen
#' @param paths - the number of simulated paths
#' @param seed - the seed of the random number generator
#' @param antithetic - if TRUE, antithetic random numbers will be chosen to
#' decrease variance
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}
#'
#' @examples Malliavin_European_Greeks(initial_price = 110,
#' exercise_price = 100, r = 0.02, time_to_maturity = 4.5,
#' volatility = 0.22, greek = c("fair_value", "delta", "rho"), payoff = "put")
#'
#' @references
#' Hudde, A., & Rüschendorf, L. (2023). European and Asian Greeks for Exponential Lévy Processes. Methodol Comput Appl Probab, 25 (39). \doi{10.1007/s11009-023-10014-5}
#'

Malliavin_European_Greeks <-
  function(initial_price = 100,
           exercise_price = 100,
           r = 0,
           time_to_maturity = 1,
           volatility = 0.3,
           payoff = "call",
           greek = c("fair_value", "delta", "vega", "theta", "rho", "gamma"),
           model = "Black Scholes",
           paths = 10000,
           seed = 1,
           antithetic = FALSE) {

    result <- vector(mode = "numeric", length = length(greek)) * NA

    names(result) <- greek

    ## the seed is set

    if (!is.na(seed)) {
      dqset.seed(seed)
    }

    ## the increments of the Brownian motion ###

    if (antithetic == TRUE) {
      W_T <- dqrnorm(n = paths/2, sd = sqrt(time_to_maturity))
      W_T <- rbind(W_T, -W_T)
    } else {
      W_T <- dqrnorm(n = paths, sd = sqrt(time_to_maturity))
    }

    ### the payoff function ###

    if (inherits(payoff, "function")) {
      print("custom payoff")
    } else if (payoff == "call") {
      payoff <- function(x) {
        return(pmax(0, x-exercise_price))
      }
    } else if (payoff == "put") {
      payoff <- function(x) {
        return(pmax(0, exercise_price-x))
      }
    } else if (payoff %in% c("digital_call", "cash_or_nothing_call")) {
      payoff <- function(x) {ifelse(x >= exercise_price, 1, 0)
      }
    } else if (payoff %in% c("digital_put", "cash_or_nothing_put")) {
      payoff <- function(x) {ifelse(x <= exercise_price, 1, 0)
      }
    } else if (payoff %in% c("asset_or_nothing_call")) {
      payoff <- function(x) {ifelse(x >= exercise_price, x, 0)
      }
    } else if (payoff %in% c("asset_or_nothing_put")) {
      payoff <- function(x) {ifelse(x <= exercise_price, x, 0)
      }
    }

    if (model == "Black Scholes") {
      X_T <- initial_price *
        exp((r - (volatility^2)/2)*time_to_maturity +
              (volatility*W_T))
    } else {
      stop("Unknown model")
    }

    E <- function(weight) {
      return(exp(-r*time_to_maturity) *
               mean(payoff(X_T) * weight))
    }

    if ("fair_value" %in% greek) {
      result["fair_value"] <-
        E(1)
    }

    if ("delta" %in% greek) {
      result["delta"] <-
        (W_T / (initial_price * volatility * time_to_maturity)) %>%
        E()
    }

    if ("vega" %in% greek) {
      result["vega"] <-
        (W_T^2/(volatility*time_to_maturity) - W_T - 1/volatility) %>%
        E()
    }

    if ("rho" %in% greek) {
      result["rho"] <-
        (time_to_maturity * (W_T/(volatility*time_to_maturity) - 1)) %>%
        E()
    }

    if ("theta" %in% greek) {
      result["theta"] <-
        -(W_T^2/(2*time_to_maturity^2) +
            (r - volatility^2/2)*W_T/(volatility*time_to_maturity) -
            (1/(2*time_to_maturity) + r)) %>%
        E()
    }

    if ("gamma" %in% greek) {
      result["gamma"] <-
        ((1 / (initial_price^2 * volatility * time_to_maturity)) *
           (W_T^2/(volatility*time_to_maturity) - W_T - 1/volatility)) %>%
        E()
    }

    return(result)

  }

test_that("Malliavin_European_Greeks is correct", {

  number_of_runs <- 6

  Greeks <- c("fair_value", "delta", "vega", "theta", "rho", "gamma")

  error <- matrix(nrow = number_of_runs, ncol = length(Greeks))

  set.seed(42)

  epsilon <- 1e-5

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 6)
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- c("call", "put", "cash_or_nothing_call", "cash_or_nothing_put",
                "asset_or_nothing_call", "asset_or_nothing_put")[i]
    antithetic <- sample(c(TRUE, FALSE), 1)

    Malliavin_Value <-
      Malliavin_European_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        payoff = payoff,
        greek = Greeks,
        paths = 2000000,
        antithetic = antithetic
      )

    Exact_Value <-
      BS_European_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = 0,
        payoff = payoff,
        greek = Greeks
      )

    error[i, ] <-
      pmin(abs(Malliavin_Value - Exact_Value)/(abs(Malliavin_Value + epsilon)),
           abs(Malliavin_Value - Exact_Value))

  }

  # "asset_or_nothing_call and asset_or_nothing_put payoff-functions have much
  # more variance
  expect(max(error[1:4]) < 0.01 && max(error[5:6]) < 0.1,
         "The results of Malliavin_European_Greeks() are not close enough to
         BS_European_Greeks()")

  expect_error(Malliavin_European_Greeks(model = "whatever_model"))

  # Check, whether custom payoff function works

  call_function <- function(x) {
    return(pmax(0, x-100))
  }

  diff <-
    sum(abs(Malliavin_European_Greeks(payoff = call_function) -
              Malliavin_European_Greeks(payoff = "call")))

  expect(abs(diff) < 1e-7, "Custom payoff function does not seem to work")

})

test_that("Binomial_American_Greeks is correct", {

  # We check the Greeks by also computing the derivative with finite difference
  # and comparing the results

  number_of_runs <- 50

  definition_of_greeks <-
    data.frame(greek = "delta", start = "fair_value", param = "initial_price") %>%
    add_row(greek = "epsilon", start = "fair_value", param = "dividend_yield") %>%
    add_row(greek = "gamma", start = "delta", param = "initial_price") %>%
    add_row(greek = "rho", start = "fair_value", param = "r") %>%
    add_row(greek = "theta", start = "fair_value", param = "time_to_maturity") %>%
    add_row(greek = "vega", start = "fair_value", param = "volatility")

  error <- numeric(number_of_runs)

  set.seed(42)

  epsilon <- 1e-5

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 6)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- sample(c("call", "put"), 1)
    greek <- sample(definition_of_greeks$greek, 1)
    param <-
      definition_of_greeks[definition_of_greeks$greek == greek, "param"] %>%
      as.character()
    start <-
      definition_of_greeks[definition_of_greeks$greek == greek, "start"] %>%
      as.character()

    Vals <-
      Greeks(
        option_type = "American",
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = greek
      )

    ## theta is minus the derivative of fair_value w.r.t. time_to_maturity
    if (greek == "theta") {
      Vals = -Vals
    }

    F <- function(epsilon) {
      assign(param, get(param) + epsilon)
      Greeks(
        option_type = "American",
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = start
      )
    }

    Vals_fd <- (F(epsilon) - F(-epsilon)) / (2 * epsilon)

    ## lambda is delta * initial_price / fair_value
    if(greek == "lambda") {
      Vals_fd <- Vals_fd *  initial_price /
        Greeks(
          option_type = "American",
          initial_price = initial_price,
          exercise_price = exercise_price,
          r = r,
          time_to_maturity = time_to_maturity,
          volatility = volatility,
          dividend_yield = dividend_yield,
          payoff = payoff,
          greek = "fair_value"
        )
    }

    error[i] <-
      min(abs(Vals - Vals_fd)/(abs(Vals + epsilon)),
          abs(Vals - Vals_fd))

  }

  expect(
    max(error) < 0.1,
    failure_message = "The results of Binomial_American_Greeks.R cannot be
    confirmend by finite difference")

})

test_that("Binomial_American_Greeks fair_value is correct", {

  # We compare the values of Binomial_Amerian_Greeks.cpp with the values of
  # Binomial_Americian_Greeks_test

  Binomial_American_Greeks_test <-
    function(initial_price = 50,
             exercise_price = 50,
             r = 0.1,
             time_to_maturity = 5/12,
             volatility = 0.4,
             dividend_yield = 0,
             payoff = "put",
             steps = 5) {

      ## the payoff function ##

      if (payoff == "call") {
        payoff <- function(x, exercise_price) {
          return(max(0, x - exercise_price))
        }
      } else if (payoff == "put") {
        payoff <- function(x, exercise_price) {
          return(max(0, exercise_price - x))
        }
      }

      underlying <- matrix(NA, nrow = steps + 1, ncol = steps + 1)
      american_option_value <- matrix(NA, nrow = steps + 1, ncol = steps + 1)
      european_option_value <- matrix(NA, nrow = steps + 1, ncol = steps + 1)

      # dt is the length of the time step
      dt <- time_to_maturity/steps
      # size of one step up or down
      up <- exp(volatility * sqrt(dt))
      down <- exp(-volatility * sqrt(dt))
      # p is the probability of going one step up
      p <- (exp((r-dividend_yield)*dt) - down) / (up - down)
      # the tree is generated
      underlying[1, 1] <- initial_price

      for (j in 2:(steps+1)) {
        underlying[1, j] <- up * underlying[1, j-1]
        for (i in 2:j) {
          underlying[i, j] <- down * underlying[i-1, j-1]
        }
      }
      # initializing with the prices
      for(i in 1:(steps+1)) {
        american_option_value[i, steps + 1] <- exp(-r*steps*dt) * payoff(underlying[i, steps + 1], exercise_price)
        european_option_value[i, steps + 1] <- exp(-r*steps*dt) * payoff(underlying[i, steps + 1], exercise_price)
      }

      for (j in steps:1) {
        for (i in 1:j) {
          american_option_value[i, j] <-
            max(
              (american_option_value[i, j+1] * p + american_option_value[i+1, j+1] * (1-p)),
              exp(-r*(j-1)*dt) * payoff(underlying[i, j], exercise_price))
          european_option_value[i, j] <-
            (european_option_value[i, j+1] * p + european_option_value[i+1, j+1] * (1-p)) * exp(-r*dt)
        }
      }

      return(american_option_value[1, 1])

    }

  number_of_runs <- 50

  error <- numeric(number_of_runs)

  set.seed(42)

  epsilon <- 1e-9

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 6)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.01, 1)
    payoff <- sample(c("call", "put"), 1)

    error[i] <- abs(
      Binomial_American_Greeks_cpp(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        steps = 5
      )[1] -
        Binomial_American_Greeks_test(
          initial_price = initial_price,
          exercise_price = exercise_price,
          r = r,
          time_to_maturity = time_to_maturity,
          volatility = volatility,
          dividend_yield = dividend_yield,
          payoff = payoff,
          steps = 5
        )
    )

  }
  expect(max(error) < 1e-5,
         failure_message = "The results of Binomial_American_Greeks.R cannot be
    confirmend by Binomial_Americian_Greeks_test")

})

test_that("Binomial_American_Greeks rejects invalid payoff input", {
  expect_error(
    Binomial_American_Greeks(payoff = "foo", steps = 3),
    "payoff must be either 'call' or 'put'"
  )

  expect_error(
    Binomial_American_Greeks_cpp(payoff = "foo", steps = 3),
    "payoff must be either 'call' or 'put'"
  )
})

#' @title Computes the Greeks of various options in the Black Scholes model or
#' both in the Black Scholes model or a Jump Diffusion model in the case of Asian
#' Options, or in the Binomial options pricing model
#'
#' @description Greeks are derivatives of the option value with respect to
#' underlying parameters.
#' For instance, the Greek
#' \eqn{\Delta = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}
#' (Delta) measures how the price of an option changes with a minor change in
#' the underlying asset's price, while
#' \eqn{\Gamma = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}
#' (Gamma) measures how \eqn{\Delta} itself changes as the price of the
#' underlying asset shifts.
#' Greeks can be computed for different types of options:
#' For
#' - **European Greeks** see also [BS_European_Greeks] and [Malliavin_European_Greeks]
#' - **American Greeks** see also [Binomial_American_Greeks]
#' - **Asian Greeks** see also [BS_Malliavin_Asian_Greeks] and [Malliavin_Asian_Greeks]
#' - **Geometric Asian Greeks** see also [BS_Geometric_Asian_Greeks] and [Malliavin_Asian_Greeks]
#'
#' The Greeks are defined as the following partial derivatives of the option
#' value:
#' - \code{Delta} = \eqn{\Delta = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}, the derivative with respect to the price of the
#' underlying asset
#' - \code{Vega} = \eqn{\mathcal{V} = \frac{\partial \text{fair\_value}}{\partial \text{volatility}}},
#' the derivative with respect to the volatility
#' - \code{Theta} = \eqn{\Theta = -\frac{\partial \text{fair\_value}}{\partial \text{time\_to\_maturity}}},
#' the negative derivative with respect to the time until expiration of the
#' option
#' - \code{rho} = \eqn{\rho = \frac{\partial \text{fair\_value}}{\partial r}},
#' the derivative with respect to the risk-free interest rate
#' - \code{Epsilon} = \eqn{\epsilon = \frac{\partial \text{fair\_value}}{\partial \text{time\_to\_maturity}}},
#' the derivative with respect to the dividend yield of the underlying asset
#' - \code{Lambda} = \eqn{\lambda = \Delta \times \frac{\text{initial\_price}}{\text{exercise\_price}}}
#' - \code{Gamma} = \eqn{\Gamma = \frac{\partial^2 \text{fair\_value}}{\partial \text{initial\_price}^2}}, the second derivative with respect to the price of
#' the underlying asset
#' - \code{Vanna} = \eqn{\frac{\partial \Delta}{\partial \text{volatility}} = \frac{\partial^2 \text{fair\_value}}{\partial \text{intial\_price} \, \partial \text{volatility}}}, the derivative of \eqn{\Delta} with respect to the volatility
#' - \code{Vomma} = \eqn{\frac{\partial^2 \text{fair\_value}}{\partial \text{volatility}^2}}, the second derivative with respect to the volatility
#' - \code{Veta} = \eqn{
#' \frac{\partial \mathcal V}{\partial r}
#' = \frac{\partial^2 \text{fair\_value}}{\partial \text{volatility} \, \partial \text{time\_to\_maturity}}},
#' the derivative of \eqn{\mathcal V} with respect to the time until expiration
#' of the option
#' - \code{Vera} = \eqn{\frac{\partial^2 \text{fair\_value}}{\partial \text{volatiliy} \, \partial \text{r}}},
#' the derivative of \eqn{\mathcal V} with respect to the risk-free interest rate
#' - \code{Speed} = \eqn{\frac{\partial \Gamma}{\partial \text{initial\_price}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{initial\_price}^3}},
#' the third derivative of the option value with respect to the price of the
#' underlying asset
#' - \code{Zomma} = \eqn{\frac{\Gamma}{\text{volatility}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{volatility}^3}},
#' the derivative of Gamma with respect to the volatility
#'- \code{Color} = \eqn{\frac{\partial \Gamma}{\partial \text{r}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{initial\_price}^2 \partial \text{r}}},
#' the derivative of Gamma with respect to the risk-free interest rate
#' - \code{Ultima} = \eqn{\frac{\partial \text{Vomma}}{\partial \text{volatility}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{volatility}^3}},
#' the third derivative with respect to the volatility
#'
#' [Greeks] computes Greeks for the following option types:
#' - **European put- and call options**, which give to option holder the right
#' but not the obligation to sell (resp. buy) the underlying asset for a
#' specific price at a specific date.
#' If $K$ is the exercise price, and \eqn{S_T} the value of the underlying asset
#' at time-to-maturity \eqn{T}, a European options pay off the following amount
#' at expiration:
#'   - \eqn{\max\{K - S_T, 0\}} for a **put-option**
#'   - \eqn{\max\{S_T - K, 0\}} for a **call-option**
#' - **American put- and call options** are like European options, but allow
#' the holder to exercise at any time until expiration
#' - **European cash-or-nothing put- and call options** provide the holder with
#' a fixed amount of cash, if the value of the underlying asset is below (resp.
#' above) a certain strike price
#' - **European asset-or-nothing put- and call options** are similar to
#' cash-or-nothing options, but provide the holder with one share of the asset.
#' - **Asian put- and call options** have a similar payoff to European put- and
#' call options but differ from European options in that they are path dependent.
#' Not the price \eqn{S_T} of the underlying asset at time-to-maturity \eqn{T}
#' is evaluated, but the arithmetic average
#' \eqn{\frac{1}{T} \int_0^T S_t dt}.
#' We get the payoffs
#'   - \eqn{\max\{K - \frac{1}{T} \int_0^T S_t dt, 0\}} for an Asian
#'   **put-option**
#'   - \eqn{\max\{\frac{1}{T} \int_0^T S_t dt - K, 0\}} for an Asian
#'   **call-option**
#' - **Geometric Asian options** differ from Asian options in that the geometric
#' average
#' \eqn{\exp \left( \frac{1}{T} \int_0^T \ln S_t dt \right)} is evaluated.
#'
#' For reference see Hull (2022) or
#'
#' [en.wikipedia.org/wiki/Greeks_(finance)](https://en.wikipedia.org/wiki/Greeks_(finance)).
#'
#' @export
#'
#' @seealso [BS_European_Greeks] for option_type = "European"
#' @seealso [Binomial_American_Greeks] for option_type = "American"
#' @seealso [BS_Geometric_Asian_Greeks] for option_type = = "Geometric Asian"
#' and model = "black_scholes"
#' @seealso [BS_Malliavin_Asian_Greeks] for option_type = = "Asian"
#' and model = "black_scholes" and greek in c("fair_value", "delta", "rho",
#' "vega")
#' @seealso [Malliavin_Asian_Greeks] for more general cases of Asian Greeks
#' @seealso [Greeks_UI] for an interactive visualization
#'
#' @param initial_price - initial price of the underlying asset
#' @param exercise_price - strike price of the option
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param dividend_yield - dividend yield
#' @param volatility - volatility of the underlying asset
#' @param model - the model to be chosen in ("black_scholes", "jump_diffusion")
#' @param option_type in c("European", "American", "Asian", "Geometric Asian",
#' "Digital", "Binomial) - the type of option to be considered
#' @param payoff - in c("call", "put", "cash_or_nothing_call",
#' "cash_or_nothing_put", "asset_or_nothing_call", "asset_or_nothing_put")
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "vega",
#' "theta", "rho", "epsilon", "lambda", "gamma", "vanna", "charm", "vomma",
#' "veta", "vera", "speed", "zomma", "color", "ultima")
#' @param ... - ...	Other arguments passed on to methods
#' @param antithetic - if TRUE, antithetic random numbers will be chosen to
#' decrease variance
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples
#' Greeks(initial_price = 100, exercise_price = 120, r = 0.01,
#' time_to_maturity = 5, volatility = 0.30, payoff = "call")
#'
#' Greeks(initial_price = 100, exercise_price = 100, r = -0.005,
#' time_to_maturity = 1, volatility = 0.30, payoff = "put",
#' option_type = "American")
#'
#' @references
#' Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition). Pearson
#'
#' [en.wikipedia.org/wiki/Greeks_(finance)](https://en.wikipedia.org/wiki/Greeks_(finance))

Greeks <-
  function(initial_price,
           exercise_price,
           r,
           time_to_maturity,
           volatility,
           dividend_yield = 0,
           model = "Black_Scholes",
           option_type = "European",
           payoff = "call",
           greek = c("fair_value", "delta", "vega", "theta", "rho", "gamma"),
           antithetic = TRUE,
           ...){

    if (tolower(option_type) %in% c("digital", "european") &&
        tolower(model) == "black_scholes") {
      if (tolower(option_type) == "digital" &&
          tolower(payoff) %in% c("call", "put")) {
        stop("Please choose one of the following:
                'cash_or_nothing_call',
                'cash_or_nothing_put',
                'asset_or_nothing_call',
                'asset_or_nothing_put'.")
      }
      return(BS_European_Greeks(payoff = payoff,
                                greek = greek,
                                initial_price = initial_price,
                                exercise_price = exercise_price,
                                r = r,
                                time_to_maturity = time_to_maturity,
                                volatility = volatility,
                                dividend_yield = dividend_yield))
    }

    else if (tolower(option_type) == "american" && tolower(model) == "black_scholes") {
      return(Binomial_American_Greeks(payoff = payoff,
                                      greek = greek,
                                      initial_price = initial_price,
                                      exercise_price = exercise_price,
                                      r = r,
                                      time_to_maturity = time_to_maturity,
                                      volatility = volatility,
                                      dividend_yield = dividend_yield,
                                      ...))
    }

    else if (tolower(option_type) == "geometric asian" && tolower(model) == "black_scholes") {
      return(BS_Geometric_Asian_Greeks(payoff = payoff,
                                       greek = greek,
                                       initial_price = initial_price,
                                       exercise_price = exercise_price,
                                       r = r,
                                       time_to_maturity = time_to_maturity,
                                       volatility = volatility,
                                       dividend_yield = dividend_yield))
    }

    else if (tolower(option_type) == "asian" && tolower(model) == "black_scholes" &&
             all(greek %in% c("fair_value", "delta", "rho", "vega"))) {
      return(BS_Malliavin_Asian_Greeks(payoff = payoff,
                                       greek = greek,
                                       initial_price = initial_price,
                                       exercise_price = exercise_price,
                                       r = r,
                                       time_to_maturity = time_to_maturity,
                                       volatility = volatility,
                                       dividend_yield = dividend_yield))
    }

    else if (tolower(option_type) == "asian") {
      return(Malliavin_Asian_Greeks(payoff = payoff,
                                    greek = greek,
                                    initial_price = initial_price,
                                    exercise_price = exercise_price,
                                    r = r,
                                    time_to_maturity = time_to_maturity,
                                    volatility = volatility,
                                    dividend_yield = dividend_yield,
                                    model = model,
                                    antithetic = antithetic))
    }

    else {
      stop("Wrong input or not yet implemented.")
    }

  }

test_that("Greeks is correct", {

  expect_error(
    Greeks(
      initial_price = 101,
      exercise_price = 99,
      r = 0.01,
      time_to_maturity = 13/12,
      volatility = 0.3,
      dividend_yield = 0,
      model = "Black_Scholes",
      payoff = "call",
      option_type = "digital"
    )
  )

  expect_error(
    Greeks(
      initial_price = 101,
      exercise_price = 99,
      r = 0.01,
      time_to_maturity = 13/12,
      volatility = 0.3,
      dividend_yield = 0,
      model = "Black_Scholes",
      payoff = "call",
      option_type = "lookback"
    )
  )

})

test_that("BS_Malliavin_Asian_Greeks is correct", {

  definition_of_greeks <-
    data.frame(greek = "delta", start = "fair_value", param = "initial_price") %>%
    add_row(greek = "rho", start = "fair_value", param = "r") %>%
    add_row(greek = "vega", start = "fair_value", param = "volatility")

  # We check the Greeks by also computing the derivative with finite difference
  # and comparing the results

  # We start with checking delta

  number_of_runs <- 10

  error <- numeric(number_of_runs)

  set.seed(42)

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 2)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- rep(c("call", "put"), number_of_runs)[i]
    greek <- "delta"
    param <- "initial_price"
    start <- "fair_value"

    Vals <-
      Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        model = "Black_Scholes",
        option_type = "Asian",
        payoff = payoff,
        greek = greek
      )

    F <- function(epsilon) {
      assign(param,
             c(get(param) + 2*epsilon, get(param) + epsilon,
               get(param) - 2*epsilon, get(param) - epsilon))
      Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        model = "Black_Scholes",
        option_type = "Asian",
        payoff = payoff,
        greek = start
      )
    }

    epsilon <- initial_price * 0.1

    F_eps <- F(epsilon)

    # We compute the derivative by Richardsons' extrapolation

    Vals_fd <- (-F_eps[1] + 8*F_eps[2] - 8*F_eps[3] + F_eps[4]) / (12 * epsilon)

    error[i] <-
      min(abs(Vals - Vals_fd)/(abs(Vals + epsilon)),
          abs(Vals - Vals_fd))

  }

  expect(max(error) < 0.1, "BS_Malliavin_Asian_Greeks: Delta error too high")

  # Now, we check rho and vega

  number_of_runs <- 4

  set.seed(42)

  error <- numeric(number_of_runs)

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, 0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 2)
    dividend_yield <- 0
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- rep(c("call", "put"), number_of_runs)[i]
    greek <- c("rho", "rho", "vega", "vega")[i]
    param <-
      definition_of_greeks[definition_of_greeks$greek == greek, "param"] %>%
      as.character()
    start <-
      definition_of_greeks[definition_of_greeks$greek == greek, "start"] %>%
      as.character()

    Vals <-
      BS_Malliavin_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = greek,
        paths = 10000,
        steps = 12
      )

    F <- function(epsilon) {
      assign(param, get(param) + epsilon)
      BS_Malliavin_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = start,
        paths = 10000,
        steps = 12
      )
    }

    epsilon <- get(param) * 0.1

    Vals_fd <- (F(epsilon) - F(-epsilon)) / (2 * epsilon)

    error[i] <-
      min(abs(Vals - Vals_fd)/(abs(Vals + epsilon)),
          abs(Vals - Vals_fd))

  }

  expect(max(error) < 0.01,
         paste0("BS_Malliavin_Asian_Greeks: ", greek, " error too high"))

})

test_that("implied volatility is correct", {

  # The implied volatility is computed with different parameters, one of them
  # being `option_price`. The resulting `implied_volatility` is the used to
  # compute the `option_price_test`. Both values should be very close.

  number_of_runs <- 4

  # the parameters are chosen at random
  set.seed(42)
  initial_price <- runif(number_of_runs, 90, 110)
  exercise_price <- runif(number_of_runs, 90, 110)
  r <- runif(number_of_runs, -0.01, 0.1)
  time_to_maturity <- runif(number_of_runs, 0.2, 6)
  dividend_yield <- runif(number_of_runs, 0, 0.1)
  model <- "Black_Scholes"
  option_type <- c("European", "European", "Geometric Asian", "Geometric Asian")
  payoff <- rep(c("call", "put"), 2)
  volatility_to_compute_initial_price <- runif(number_of_runs, 0.01, 1)

  option_price <- numeric(number_of_runs)
  option_price_zero_vol <- numeric(number_of_runs)
  option_price_test <- numeric(number_of_runs)

  implied_volatility <- numeric(number_of_runs)

  for(i in 1:number_of_runs) {
    option_price[i] <-
      Greeks(
        initial_price = initial_price[i],
        exercise_price = exercise_price[i],
        r = r[i],
        time_to_maturity = time_to_maturity[i],
        volatility = volatility_to_compute_initial_price[i],
        dividend_yield = dividend_yield[i],
        option_type = option_type[i],
        payoff = payoff[i],
        greek = "fair_value"
      )

    option_price_zero_vol[i] <-
      Greeks(
        initial_price = initial_price[i],
        exercise_price = exercise_price[i],
        r = r[i],
        time_to_maturity = time_to_maturity[i],
        volatility = 1e-12,
        dividend_yield = dividend_yield[i],
        option_type = option_type[i],
        payoff = payoff[i],
        greek = "fair_value"
      )

    # We check if the implied volatility for the chosen parameters is
    # well-defined
    if (option_price[i] <= option_price_zero_vol[i] + 1e-12) {
      option_price[i] <- 0
      next
    }

    implied_volatility[i] <-
      Implied_Volatility(
        option_price = option_price[i],
        initial_price = initial_price[i],
        exercise_price = exercise_price[i],
        r = r[i],
        time_to_maturity = time_to_maturity[i],
        dividend_yield = dividend_yield[i],
        model = model,
        option_type = option_type[i],
        payoff = payoff[i]
      )

    option_price_test[i] <-
      Greeks(
        initial_price = initial_price[i],
        exercise_price = exercise_price[i],
        r = r[i],
        time_to_maturity = time_to_maturity[i],
        volatility = implied_volatility[i],
        dividend_yield = dividend_yield[i],
        model = model,
        option_type = option_type[i],
        payoff = payoff[i],
        greek = "fair_value"
      )

  }

  expect(max(abs(option_price_test - option_price)) < 1e-06,
  "Implied_Volatility error is too high")

  # Checks if error is caught when the option price is infeasible low
  expect_error(Implied_Volatility(option_price = 10, exercise_price = 90))

})

#' @title
#' Computes the Greeks of a geometric Asian option with the Malliavin Monte
#' Carlo Method in the Black Scholes- or Jump diffusion model
#'
#' @description
#' In contrast to Asian options (see [Malliavin_Asian_Greeks]), geometric Asian
#' options evaluate the geometric average
#' \eqn{\exp \left( \frac{1}{T} \int_0^T \ln S_t dt \right)}, where
#' \eqn{S_t} is the price of the underlying asset at time \eqn{t} and \eqn{T} is
#' the time-to-maturity of the option (see
#'
#' [en.wikipedia.org/wiki/Asian_option#European_Asian_call_and_put_options_with_geometric_averaging](https://en.wikipedia.org/wiki/Asian_option#European_Asian_call_and_put_options_with_geometric_averaging)).
#' For more details on the definition of Greeks see [Greeks], and for a
#' description of the Malliavin Monte Carlo Method for Greeks see for example
#' (Hudde & Rüschendorf, 2023).
#'
#' @export
#'
#' @seealso [BS_Geometric_Asian_Greeks] for exact and fast computation in the
#' Black Scholes model and for put- and call payoff functions
#'
#' @import "stats"
#' @import "Rcpp"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset, can also be a
#' vector
#' @param exercise_price - strike price of the option, can also be a vector
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param volatility - volatility of the underlying asset
#' @param dividend_yield - dividend yield
#' @param payoff - the payoff function, either a string in ("call", "put",
#' "digital_call", "digital_put"), or a function
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "rho", "vega", "theta", "gamma)
#' @param model - the model to be chosen in ("black_scholes", "jump_diffusion")
#' @param lambda - the lambda of the Poisson process in the jump-diffusion model
#' @param alpha - the alpha in the jump-diffusion model influences the jump size
#' @param jump_distribution - the distribution of the jumps, choose a function
#' which generates random numbers with the desired distribution
#' @param steps - the number of integration steps
#' @param paths - the number of simulated paths
#' @param seed - the seed of the random number generator
#' @param antithetic - if TRUE, antithetic random numbers will be chosen to
#' decrease variance
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples Malliavin_Asian_Greeks(initial_price = 110, exercise_price = 100,
#' r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
#' greek = c("fair_value", "delta", "rho"), payoff = "put")
#'
#' @references
#' Hudde, A., & Rüschendorf, L. (2023). European and Asian Greeks for Exponential Lévy Processes. Methodol Comput Appl Probab, 25 (39). \doi{10.1007/s11009-023-10014-5}
#'

Malliavin_Geometric_Asian_Greeks <- function(
    initial_price = 100,
    exercise_price = 100,
    r = 0,
    time_to_maturity = 1,
    volatility = 0.3,
    dividend_yield = 0,
    payoff = "call",
    greek = c("fair_value", "delta", "rho", "vega",
              "theta", "gamma"),
    model = "black_scholes",
    lambda = 0.2,
    alpha = 0.3,
    jump_distribution = function(n) stats::rt(n, df = 3),
    steps = round(time_to_maturity*252),
    paths = 10000,
    seed = 1,
    antithetic = FALSE) {

  params <- c("initial_price", "exercise_price", "r", "time_to_maturity",
              "volatility", "dividend_yield")

  param <- params[1]
  vectorized_param <- get(param)

  for (p in params) {
    if ( length(get(p)) >= 2) {
      vectorized_param <- get(p)
      param <- p
      break
    }
  }

  dt <- time_to_maturity/steps

  result <-
    matrix(ncol = length(greek),
           nrow = length(vectorized_param),
           dimnames = list(NULL, greek)) * NA

  # the payoff function
  if (inherits(payoff, "function")) {
    print("custom payoff")
  } else if (payoff == "call") {
    payoff <- function(x, exercise_price) {
      return(pmax(0, x - exercise_price))
    }
  } else if (payoff == "put") {
    payoff <- function(x, exercise_price) {
      return(pmax(0, exercise_price - x))
    }
  } else if (payoff == "digital_call") {
    payoff <- function(x, exercise_price) {ifelse(x >= exercise_price, 1, 0)
    }
  } else if (payoff == "digital_put") {
    payoff <- function(x, exercise_price) {ifelse(x <= exercise_price, 1, 0)
    }
  }

  # the seed is set
  if (!is.na(seed)) {
    dqset.seed(seed)
  }

  W <- make_BM(dqrnorm(n = paths*steps, sd = sqrt(dt)), paths = paths, steps = steps)

  X <- calc_X(W, dt, volatility, r - dividend_yield)

  if (model == "jump_diffusion") {

    Jumps <- c(numeric(paths), rpois(n = steps * paths, lambda = lambda *
                                       dt))
    for (i in which(Jumps != 0)) {
      Jumps[i] <- alpha * sum(jump_distribution(Jumps[i]))
    }
    Jumps <- Jumps %>% matrix(nrow = paths) %>% rowCumsums()
    X <- X * exp(Jumps)

  } # model == "jump_diffusion"

  W_T <- W[, steps + 1]

  X_T <- X[, steps + 1]

  # the calculation of I_W, the integral \int_0^T W_t dt
  I_W <- calc_I(W, steps, dt)

  # the calculation of I_{(n)}, the integral \int_0^T t^n X_t dt
  I_0 <- calc_I(X, steps, dt)

  # TODO: comment
  if (length(intersect(greek, c("delta", "theta", "vega", "gamma")))) {
    I_1 <- calc_I_1(X, steps, dt)
    I_2 <- calc_I_2(X, steps, dt)
  }

  # the calculation of I_ln_X, the integral \int_0^T ln(X_t) dt
  if("theta" %in% greek) {
    I_ln_X <- calc_I(log(X), steps, dt)
  }

  for (i in 1:length(vectorized_param)) {

    assign(param, vectorized_param[i])

    # the calculation of the geometric average of X
    I_0_geom <-
      exp(calc_I(log(initial_price * X), steps, dt) / time_to_maturity)

    # the value of the greek, given the Malliavin weight
    E_I_0_geom <- function(weight) {
      return(exp(-(r - dividend_yield) * time_to_maturity) *
               mean(payoff(I_0_geom, exercise_price) * weight))
    }

    if ("fair_value" %in% greek) {
      result[i, "fair_value"] <- E_I_0_geom(1)
    } #fair_value

    if ("delta" %in% greek) {
      result[i, "delta"] <-
        (2/(initial_price*volatility*time_to_maturity)) * E_I_0_geom(W_T)
    } #delta

    if ("rho" %in% greek) {
      result[i, "rho"] <-
        (W_T/volatility - time_to_maturity) %>%
        E_I_0_geom()
    } #rho

    if ("theta" %in% greek) {
      result[i, "theta"] <-
        ((r - dividend_yield) +
           (2 / (volatility * time_to_maturity^3)) * W_T * I_ln_X -
           (1 / time_to_maturity) -
           (2 / (volatility * time_to_maturity^2)) * log(X_T) * W_T +
           (2 / time_to_maturity)) %>%
        E_I_0_geom()
    } #theta

    if ("vega" %in% greek) {
      result[i, "vega"] <-
        (2/(volatility * time_to_maturity**2) * W_T * I_W -
           1/volatility - W_T)  %>%
        E_I_0_geom()

    } #vega

    if ("gamma" %in% greek) {
      result[i, "gamma"] <-
        ((-(1/initial_price) * (2/(initial_price*volatility*time_to_maturity)) * W_T) +
           (4/(initial_price^2 * volatility^2 * time_to_maturity) * (W_T^2 - time_to_maturity))) %>%
        E_I_0_geom()
    } #gamma

  }

  return(drop(result))

}

test_that("Malliavin_Geometric_Asian_Greeks is correct", {

  # We check the Greeks by comparing with the exact results
  number_of_runs <- 8

  Greeks <- c("fair_value", "delta", "theta", "rho", "gamma")

  error <- matrix(nrow = number_of_runs, ncol = length(Greeks))

  set.seed(42)

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 1.5)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- rep(c("put", "call"), number_of_runs/2)[i]
    antithetic <- sample(c(TRUE, FALSE), 1)

    Value_MC <-
      Malliavin_Geometric_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = Greeks,
        paths = 1000000,
        steps = 48,
        antithetic = antithetic
      )

    Value_exact <-
      BS_Geometric_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = Greeks
      )

    for(j in 1:length(Greeks)) {
      error[i, j] <-
      min(abs(Value_MC[j] - Value_exact[j])/(abs(Value_MC[j]) + 1e-5),
          abs(Value_MC[j] - Value_exact[j]))
    }

  }

  expect(
    max(error) < 0.1,
    failure_message = "The results of Malliavin_Geometric_Asian_Greeks.R cannot
    be confirmend by BS_Geometric_Asian_Greeks.R")

  # We check, whether computation for vectorized parameters initial_value and
  # exercise price works

  vectorized_initial_price <-
    Malliavin_Geometric_Asian_Greeks(
      initial_price = 99:101,
      exercise_price = exercise_price,
      r = r,
      time_to_maturity = time_to_maturity,
      volatility = volatility,
      dividend_yield = dividend_yield,
      payoff = payoff,
      greek = Greeks,
      paths = 100)

  single_initial_price <-
    Malliavin_Geometric_Asian_Greeks(
      initial_price = 100,
      exercise_price = exercise_price,
      r = r,
      time_to_maturity = time_to_maturity,
      volatility = volatility,
      dividend_yield = dividend_yield,
      payoff = payoff,
      greek = Greeks,
      paths = 100)

  expect(max(abs(vectorized_initial_price[2, ] - single_initial_price)) < 1e-9,
         failure_message = "Malliavin_Geometric_Asian_Greeks: Vectorized
         computation wrt to initial_value does not work")

  vectorized_exercise_price <-
    Malliavin_Geometric_Asian_Greeks(
      initial_price = initial_price,
      exercise_price = 99:101,
      r = r,
      time_to_maturity = time_to_maturity,
      volatility = volatility,
      dividend_yield = dividend_yield,
      payoff = payoff,
      greek = Greeks,
      paths = 100)

  single_exercise_price <-
    Malliavin_Geometric_Asian_Greeks(
      initial_price = initial_price,
      exercise_price = 100,
      r = r,
      time_to_maturity = time_to_maturity,
      volatility = volatility,
      dividend_yield = dividend_yield,
      payoff = payoff,
      greek = Greeks,
      paths = 100)

  expect(
    max(abs(vectorized_exercise_price[2, ] - single_exercise_price)) < 1e-9,
    failure_message =  "Malliavin_Geometric_Asian_Greeks: Vectorized computation
    wrt to exercise_price does not work")

  # We check, whether custom payoff functions work

  digital_call <-
    function(x, exercise_price) {ifelse(x >= exercise_price, 1, 0)}

  expect(
    max(abs(
      Malliavin_Geometric_Asian_Greeks(payoff = digital_call, paths = 100) -
        Malliavin_Geometric_Asian_Greeks(payoff = "digital_call", paths = 100))) < 1e-9,
    failure_message =  "Malliavin_Geometric_Asian_Greeks: Custom payoff
    functions do not work")

  digital_put <-
    function(x, exercise_price) {ifelse(x <= exercise_price, 1, 0)}

  expect(max(abs(
    Malliavin_Geometric_Asian_Greeks(payoff = digital_put, paths = 100) -
      Malliavin_Geometric_Asian_Greeks(payoff = "digital_put", paths = 100))) < 1e-9,
    failure_message =  "Malliavin_Geometric_Asian_Greeks: Custom payoff
    functions do not work")

})

#' @title
#' Computes the Greeks of an Asian option with the Malliavin Monte Carlo
#' Method in the Black Scholes model, or for Asian options, also in a Jump
#' Diffusion model
#'
#' @description
#' Asian options are path-dependent.
#' If \eqn{S_t} is the price of the underlying asset at time \eqn{t}, the
#' execution of an Asian option depends on the average price of option,
#' \eqn{\frac{1}{T} \int_0^T S_t dt}, where \eqn{T} is the time-to-maturity of
#' the option.
#' For more details on the definition of Greeks in general see [Greeks].
#'
#' For a description of Malliavin Monte Carlo Methods for Greeks see for example
#' (Hudde & Rüschendorf, 2023).
#'
#' @export
#'
#' @seealso [BS_Malliavin_Asian_Greeks] for a faster computation, but only in
#' the Black Scholes model and with a smaller set of Greeks
#'
#' @import "stats"
#' @import "Rcpp"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset, can also be a
#' vector
#' @param exercise_price - strike price of the option, can also be a vector
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param volatility - volatility of the underlying asset
#' @param dividend_yield - dividend yield
#' @param payoff - the payoff function, either a string in ("call", "put",
#' "digital_call", "digital_put"), or a function
#' @param greek - the Greek to be calculated
#' @param model - the model to be chosen in ("black_scholes", "jump_diffusion")
#' @param lambda - the lambda of the Poisson process in the jump-diffusion model
#' @param alpha - the alpha in the jump-diffusion model influences the jump size
#' @param jump_distribution - the distribution of the jumps, choose a function
#' which generates random numbers with the desired distribution
#' @param steps - the number of integration steps
#' @param paths - the number of simulated paths
#' @param seed - the seed of the random number generator
#' @param antithetic - if TRUE, antithetic random numbers will be chosen to
#' decrease variance
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples Malliavin_Asian_Greeks(initial_price = 110, exercise_price = 100,
#' r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
#' greek = c("fair_value", "delta", "rho"), payoff = "put")
#'
#' @references
#' Hudde, A., & Rüschendorf, L. (2023). European and Asian Greeks for Exponential Lévy Processes. Methodol Comput Appl Probab, 25 (39). \doi{10.1007/s11009-023-10014-5}
#'

Malliavin_Asian_Greeks <- function(
    initial_price = 100,
    exercise_price = 100,
    r = 0,
    time_to_maturity = 1,
    volatility = 0.3,
    dividend_yield = 0,
    payoff = "call",
    greek = c("fair_value", "delta", "rho", "vega",
              "theta", "gamma"),
    model = "black_scholes",
    lambda = 0.2,
    alpha = 0.3,
    jump_distribution = function(n) stats::rt(n, df = 3),
    steps = round(time_to_maturity*252),
    paths = 10000,
    seed = 1,
    antithetic = FALSE) {

  params <- c("initial_price", "exercise_price", "r", "time_to_maturity",
              "volatility", "dividend_yield")

  param <- params[1]
  vectorized_param <- get(param)

  for (p in params) {
    if ( length(get(p)) >= 2) {
      vectorized_param <- get(p)
      param <- p
      break
    }
  }

  dt <- time_to_maturity/steps

  result <-
    matrix(ncol = length(greek),
           nrow = length(vectorized_param),
           dimnames = list(NULL, greek)) * NA

  ## the payoff function ##

  if (inherits(payoff, "function")) {
    print("custom payoff")
  } else if (payoff == "call") {
    payoff <- function(x, exercise_price) {
      return(pmax(0, x - exercise_price))
    }
    dpayoff <- function(x, exercise_price) {
      return((x > exercise_price) + 0)
    }
  } else if (payoff == "put") {
    payoff <- function(x, exercise_price) {
      return(pmax(0, exercise_price - x))
    }
    dpayoff <- function(x, exercise_price) {
      return(-(x < exercise_price) + 0)
    }
  } else if (payoff == "digital_call") {
    payoff <- function(x, exercise_price) {ifelse(x >= exercise_price, 1, 0)
    }
  } else if (payoff == "digital_put") {
    payoff <- function(x, exercise_price) {ifelse(x <= exercise_price, 1, 0)
    }
  }

  ## the seed is set

  if (!is.na(seed)) {
    dqset.seed(seed)
  }

  if (antithetic) {
    gaussian_random_numbers <- dqrnorm(n = (paths*steps/2), sd = sqrt(dt))
    W <- make_BM(gaussian_random_numbers, paths = paths/2, steps = steps)
    W <- rbind(
      W,
      -W)
  } else {
    gaussian_random_numbers <- dqrnorm(n = paths*steps, sd = sqrt(dt))
    W <- make_BM(gaussian_random_numbers, paths = paths, steps = steps)
  }

  X <- calc_X(W, dt, volatility, r - dividend_yield)

  if (model == "jump_diffusion") {

    Jumps <- c(numeric(paths), rpois(n = steps * paths, lambda = lambda *
                                       dt))
    for (i in which(Jumps != 0)) {
      Jumps[i] <- alpha * sum(jump_distribution(Jumps[i]))
    }
    Jumps <- Jumps %>% matrix(nrow = paths) %>% rowCumsums()
    X <- X * exp(Jumps)

  } # model == "jump_diffusion"

  W_T <- W[, steps + 1]

  X_T <- X[, steps + 1]

  if (length(intersect(greek, c("vega", "vega_d")))) {
    XW <- calc_XW(X, W, steps, paths, dt)
    tXW <- calc_tXW(X, W, steps, paths, dt)
  }

  rm(W)

  ### the calculation of I_{(n)}, the integral \int_0^T t^n X_t dt ###

  I_0 <- calc_I(X, steps, dt)

  if (length(intersect(
    greek,
    c("delta", "delta_d", "theta", "vega", "vega_d", "gamma", "gamma_kombi",
      "rho_d")))) {
    I_1 <- calc_I_1(X, steps, dt)
    I_2 <- calc_I_2(X, steps, dt)
  }

  if ("gamma" %in% greek) {
    I_3 <- calc_I_3(X, steps, dt)
  }

  for (i in 1:length(vectorized_param)) {

    assign(param, vectorized_param[i])

    E <- function(weight) {
      return(exp(-(r - dividend_yield)*time_to_maturity) *
               mean(payoff(initial_price * I_0/time_to_maturity, exercise_price) * weight))
    }

    dE <- function(weight) {
      return(exp(-(r - dividend_yield)*time_to_maturity) *
               mean(dpayoff(initial_price * I_0/time_to_maturity, exercise_price) * weight))
    }

    if ("fair_value" %in% greek) {
      result[i, "fair_value"] <-
        E(1)
    }

    if ("delta" %in% greek) {
      result[i, "delta"] <-
        (1/(volatility * initial_price) *
           (-volatility + I_0/I_1*W_T + volatility*I_0*I_2/(I_1^2))) %>%
        E()
    }

    if ("delta_d" %in% greek) {
      result[i, "delta_d"] <- dE(I_0 / time_to_maturity)
    }

    if ("rho" %in% greek) {
      result[i, "rho"] <-
        (W_T/volatility - time_to_maturity) %>%
        E()
    }

    if ("rho_d" %in% greek) {
      result[i, "rho_d"] <-
        -time_to_maturity * E(1) + dE(initial_price * I_1/time_to_maturity)
    }

    if ("theta" %in% greek) {
      result[i, "theta"] <-
        ((r - dividend_yield) - 1/time_to_maturity +
           ((1/(volatility * time_to_maturity)) * I_0 * W_T -
              (1/volatility) * X_T * W_T + time_to_maturity * X_T) / I_1 +
           (1/time_to_maturity * I_0 * I_2 - I_2 * X_T) / (I_1^2)) %>%
        E()
    }

    if ("theta_d" %in% greek) {
      result[i, "theta_d"] <-
        (r - dividend_yield) * E(1) +
        dE(initial_price * (I_0/time_to_maturity^2 - X_T/time_to_maturity))
    }

    if ("vega" %in% greek) {
      result[i, "vega"] <-
        ((1 / volatility) *
           ( -(1 + volatility * W_T) +
               (W_T * XW - volatility * tXW) / I_1 +
               (volatility * XW * I_2) / I_1^2)) %>%
        E()
    }

    if ("vega_d" %in% greek) {
      result[i, "vega_d"] <-
        ((initial_price / time_to_maturity) * (XW - volatility * I_1)) %>%
        dE()
    }

    if ("gamma" %in% greek) {
      result[i, "gamma"] <-
        ((1/(volatility^2*initial_price^2)) *
           (2*volatility^2
            - 4*volatility*W_T*I_0/I_1
            + ((W_T^2 - time_to_maturity)*I_0 - 4*volatility^2*I_2)*I_0/I_1^2
            + volatility * (3*W_T*I_2 - volatility*I_3)*I_0^2/I_1^3
            + 3*volatility^2*I_0^2*I_2^2/I_1^4)) %>%
        E()
    }

    if ("gamma_kombi" %in% greek) {
      result[i, "gamma_kombi"] <-
        (1/(volatility * initial_price) *
           (-volatility + I_0/I_1*W_T + volatility*I_0*I_2/(I_1^2))) %>%
        dE()
    }

  }

  return(drop(result))

}

#' @title
#' Computes the Greeks of a Geometric Asian Option with classical Call- and
#' Put-Payoff in the Black Scholes model
#'
#' @description
#' For the definition of geometric Asian options see
#' [Malliavin_Geometric_Asian_Greeks].
#' [BS_Geometric_Asian_Greeks] offers a fast and exaction computation of
#' Geometric Asian Greeks.
#'
#' @export
#'
#' @seealso [Malliavin_Geometric_Asian_Greeks] for the Monte Carlo
#' implementation which provides digital and custom payoff functions and also
#' works for the jump diffusion model
#' @seealso [Greeks_UI] for an interactive visualization
#'
#' @import "stats"
#' @import "Rcpp"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset, can also be a
#' vector
#' @param exercise_price - strike price of the option
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param volatility - volatility of the underlying asset
#' @param dividend_yield - dividend yield
#' @param payoff - the payoff function, either a string in ("call", "put")
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "rho",
#' "vega", "theta", "gamma", "vomma")
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples BS_Geometric_Asian_Greeks(initial_price = 110, exercise_price = 100,
#' r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
#' greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma"),
#' payoff = "put")
#'

BS_Geometric_Asian_Greeks <- function(
    initial_price = 100,
    exercise_price = 100,
    r = 0,
    time_to_maturity = 1,
    volatility = 0.3,
    dividend_yield = 0,
    payoff = "call",
    greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma", "vomma")) {

  BS_Geometric_Asian_Greeks_cpp(
    # Keep the old scalar-return behavior for vector numeric inputs until the
    # validation pass handles them intentionally.
    initial_price = initial_price[1],
    exercise_price = exercise_price[1],
    r = r[1],
    time_to_maturity = time_to_maturity[1],
    volatility = volatility[1],
    dividend_yield = dividend_yield[1],
    payoff = payoff,
    greek = greek
  )
}

test_that("BS_Geometric_Asian_Greeks is correct", {

  # We check the Greeks by also computing the derivative with finite difference
  # and comparing the results

  number_of_runs <- 100

  definition_of_geeks <-
    tibble(
      greek = c("delta", "gamma", "vega", "vomma", "theta", "rho"),
      start = c("fair_value", "delta", "fair_value", "vega", "fair_value",
                "fair_value"),
      param = c("initial_price", "initial_price", "volatility", "volatility",
                "time_to_maturity", "r")
    )

  error <- numeric(number_of_runs)

  set.seed(42)

  ## TODO: epsilon is a bad variable name, since is can be confused with the
  ## Greek epsilon, also change that in test-BS_European_Greeks
  epsilon <- 1e-5

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 6)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.001, 1)
    model <- "Black_Scholes"
    payoff <- sample(c("call", "put"), 1)
    greek <- sample(definition_of_geeks$greek, 1)
    param <-
      definition_of_geeks[definition_of_geeks$greek == greek, "param"] %>%
      as.character()
    start <-
      definition_of_geeks[definition_of_geeks$greek == greek, "start"] %>%
      as.character()

    Vals <-
      BS_Geometric_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = greek
      )

    if (param == "time_to_maturity") {
      Vals = -Vals
    }

    F <- function(epsilon) {
      assign(param, get(param) + epsilon)
      BS_Geometric_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = start
      )

    }

    Vals_fd <- (F(epsilon) - F(-epsilon)) / (2 * epsilon)
    error[i] <-
      min(abs(Vals - Vals_fd)/(abs(Vals + epsilon)),
          abs(Vals - Vals_fd))
  }

  expect(max(error) < sqrt(epsilon),
  "Error in BS_Geometric_Asian_Greeks is too high")

})

#' @title Computes the Greeks of a European call- or put-option, or of digital
#' options in the Black Scholes model.
#'
#' @description
#' For details on the definition of Greeks see [Greeks].
#'
#' @export
#'
#' @seealso [Malliavin_European_Greeks] for the Monte Carlo implementation
#' @seealso [Greeks_UI] for an interactive visualization
#'
#' @import "stats"
#'
#' @param initial_price - initial price of the underlying asset
#' @param exercise_price - strike price of the option
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param dividend_yield - dividend yield
#' @param volatility - volatility of the underlying asset
#' @param payoff - in c("call", "put", "cash_or_nothing_call",
#' "cash_or_nothing_put", "asset_or_nothing_call", "asset_or_nothing_put")
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "vega",
#' "theta", "rho", "epsilon", "lambda", "gamma", "vanna", "charm", "vomma",
#' "veta", "vera", "speed", "zomma", "color", "ultima")
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples BS_European_Greeks(initial_price = 120, exercise_price = 100,
#' r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
#' greek = c("fair_value", "delta", "gamma"), payoff = "put")

BS_European_Greeks <-
  function(initial_price = 100,
           exercise_price = 100,
           r = 0,
           time_to_maturity = 1,
           volatility = 0.3,
           dividend_yield = 0,
           payoff = "call",
           greek = c("fair_value", "delta", "vega", "theta", "rho", "epsilon",
                     "lambda", "gamma", "vanna", "charm", "vomma", "veta",
                     "speed")) {

    # Keep validation in R so public error messages stay friendly.
    positive_params <- c("initial_price", "exercise_price", "time_to_maturity",
                         "volatility")

    for (param in positive_params) {
      value <- get(param)

      if (!is.numeric(value) || length(value) != 1 || is.na(value) ||
          !is.finite(value) || value <= 0) {
        stop(param, " must be a positive finite number.", call. = FALSE)
      }
    }

    finite_params <- c("r", "dividend_yield")

    for (param in finite_params) {
      value <- get(param)

      if (!is.numeric(value) || length(value) != 1 || is.na(value) ||
          !is.finite(value)) {
        stop(param, " must be a finite number.", call. = FALSE)
      }
    }

    if (anyDuplicated(greek)) {
      stop("greek must not contain duplicate values.", call. = FALSE)
    }

    valid_payoffs <- c("call", "put", "cash_or_nothing_call",
                       "cash_or_nothing_put", "asset_or_nothing_call",
                       "asset_or_nothing_put")

    if (!(payoff %in% valid_payoffs)) {
      return()
    }

    BS_European_Greeks_cpp(
      initial_price = initial_price,
      exercise_price = exercise_price,
      r = r,
      time_to_maturity = time_to_maturity,
      volatility = volatility,
      dividend_yield = dividend_yield,
      payoff = payoff,
      greek = greek
    )
  }

test_that("BS_European_Greeks is correct", {

  # We check the Greeks by also computing the derivative with finite difference
  # and comparing the results

  number_of_runs <- 300

  definition_of_greeks <-
    data.frame(greek = "charm", start = "theta", param = "initial_price") %>%
    add_row(greek = "delta", start = "fair_value", param = "initial_price") %>%
    add_row(greek = "epsilon", start = "fair_value", param = "dividend_yield") %>%
    add_row(greek = "gamma", start = "delta", param = "initial_price") %>%
    add_row(greek = "rho", start = "fair_value", param = "r") %>%
    add_row(greek = "theta", start = "fair_value", param = "time_to_maturity") %>%
    add_row(greek = "vanna", start = "delta", param = "volatility") %>%
    add_row(greek = "vega", start = "fair_value", param = "volatility") %>%
    add_row(greek = "vera", start = "rho", param = "volatility") %>%
    add_row(greek = "veta", start = "vega", param = "time_to_maturity") %>%
    add_row(greek = "vomma", start = "vega", param = "volatility") %>%
    add_row(greek = "speed", start = "gamma", param = "initial_price") %>%
    add_row(greek = "zomma", start = "vanna", param = "initial_price") %>%
    add_row(greek = "color", start = "gamma", param = "time_to_maturity") %>%
    add_row(greek = "ultima", start = "vomma", param = "volatility") %>%
    add_row(greek = "lambda", start = "fair_value", param = "initial_price")

  error <- numeric(number_of_runs)

  set.seed(42)

  epsilon <- 1e-5

  for(i in 1:number_of_runs) {

    # the parameters are chosen at random
    initial_price <- runif(1, 90, 110)
    exercise_price <- runif(1, 90, 110)
    r <- runif(1, -0.01, 0.1)
    time_to_maturity <- runif(1, 0.2, 6)
    dividend_yield <- runif(1, 0, 0.1)
    volatility <- runif(1, 0.01, 1)
    model <- "Black_Scholes"
    payoff <- sample(c("call", "put",
                       "cash_or_nothing_call", "cash_or_nothing_put",
                       "asset_or_nothing_call", "asset_or_nothing_put"), 1)
    greek <- sample(definition_of_greeks$greek, 1)
    param <-
      definition_of_greeks[definition_of_greeks$greek == greek, "param"] %>%
      as.character()
    start <-
      definition_of_greeks[definition_of_greeks$greek == greek, "start"] %>%
      as.character()

    Vals <-
      BS_European_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = greek
      )

    ## theta is minus the derivative of fair_value w.r.t. time_to_maturity
    if (greek == "theta") {
      Vals = -Vals
    }

    F <- function(epsilon) {
      assign(param, get(param) + epsilon)
      BS_European_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = start
      )
    }

    Vals_fd <- (F(epsilon) - F(-epsilon)) / (2 * epsilon)

    ## lambda is delta * initial_price / fair_value
    if(greek == "lambda") {
      Vals_fd <- Vals_fd *  initial_price /
        BS_European_Greeks(
          initial_price = initial_price,
          exercise_price = exercise_price,
          r = r,
          time_to_maturity = time_to_maturity,
          volatility = volatility,
          dividend_yield = dividend_yield,
          payoff = payoff,
          greek = "fair_value"
        )
    }

    error[i] <-
      min(abs(Vals - Vals_fd)/(abs(Vals + epsilon)),
          abs(Vals - Vals_fd))

  }

  expect(max(error) < sqrt(epsilon), "Error in BS_European_Greeks is too high")

})

test_that("BS_European_Greeks rejects invalid positive model parameters", {
  invalid_positive_params <- list(
    initial_price = 0,
    exercise_price = 0,
    time_to_maturity = 0,
    volatility = 0
  )

  for (param in names(invalid_positive_params)) {
    args <- list(greek = "fair_value")
    args[[param]] <- invalid_positive_params[[param]]

    expect_error(
      do.call(BS_European_Greeks, args),
      paste0(param, " must be a positive finite number"),
      fixed = TRUE
    )
  }

  expect_error(
    BS_European_Greeks(initial_price = c(100, 101), greek = "fair_value"),
    "initial_price must be a positive finite number",
    fixed = TRUE
  )

  expect_error(
    BS_European_Greeks(volatility = Inf, greek = "fair_value"),
    "volatility must be a positive finite number",
    fixed = TRUE
  )
})

test_that("BS_European_Greeks rejects invalid finite model parameters", {
  expect_error(
    BS_European_Greeks(r = c(0.01, 0.02), greek = "fair_value"),
    "r must be a finite number",
    fixed = TRUE
  )

  expect_error(
    BS_European_Greeks(r = Inf, greek = "fair_value"),
    "r must be a finite number",
    fixed = TRUE
  )

  expect_error(
    BS_European_Greeks(dividend_yield = NA_real_, greek = "fair_value"),
    "dividend_yield must be a finite number",
    fixed = TRUE
  )
})

test_that("BS_European_Greeks rejects duplicate greek inputs", {
  expect_error(
    BS_European_Greeks(greek = c("fair_value", "fair_value")),
    "greek must not contain duplicate values",
    fixed = TRUE
  )
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

BS_European_Greeks_cpp <- function(initial_price, exercise_price, r, time_to_maturity, volatility, dividend_yield, payoff, greek) {
    .Call(`_greeks_BS_European_Greeks_cpp`, initial_price, exercise_price, r, time_to_maturity, volatility, dividend_yield, payoff, greek)
}

BS_Geometric_Asian_Greeks_cpp <- function(initial_price, exercise_price, r, time_to_maturity, volatility, dividend_yield, payoff, greek) {
    .Call(`_greeks_BS_Geometric_Asian_Greeks_cpp`, initial_price, exercise_price, r, time_to_maturity, volatility, dividend_yield, payoff, greek)
}

Binomial_American_Greeks_cpp <- function(initial_price = 100, exercise_price = 100, r = 0, time_to_maturity = 1, volatility = 0.3, dividend_yield = 0, payoff = "call", steps = 1000L) {
    .Call(`_greeks_Binomial_American_Greeks_cpp`, initial_price, exercise_price, r, time_to_maturity, volatility, dividend_yield, payoff, steps)
}

calc_I <- function(X, steps, dt) {
    .Call(`_greeks_calc_I`, X, steps, dt)
}

calc_I_1 <- function(X, steps, dt) {
    .Call(`_greeks_calc_I_1`, X, steps, dt)
}

calc_I_2 <- function(X, steps, dt) {
    .Call(`_greeks_calc_I_2`, X, steps, dt)
}

calc_I_3 <- function(X, steps, dt) {
    .Call(`_greeks_calc_I_3`, X, steps, dt)
}

calc_X <- function(W, dt, volatility, r) {
    .Call(`_greeks_calc_X`, W, dt, volatility, r)
}

calc_XW <- function(X, W, steps, paths, dt) {
    .Call(`_greeks_calc_XW`, X, W, steps, paths, dt)
}

calc_log_X <- function(W, dt, volatility, r) {
    .Call(`_greeks_calc_log_X`, W, dt, volatility, r)
}

calc_tXW <- function(X, W, steps, paths, dt) {
    .Call(`_greeks_calc_tXW`, X, W, steps, paths, dt)
}

make_BM <- function(v, steps, paths) {
    .Call(`_greeks_make_BM`, v, steps, paths)
}

rowCumsums <- function(m) {
    .Call(`_greeks_rowCumsums`, m)
}

#' @title
#' Computes the Greeks of an Asian option with the Malliavin Monte Carlo Method
#' in the Black Scholes model
#'
#' @description
#' For a description of Asian Greeks see [Malliavin_Asian_Greeks].
#' BS_Malliavin_Asian_Greeks offers a fast implementation in the Black Scholes
#' model.
#'
#' @export
#'
#' @seealso [Malliavin_Asian_Greeks] for a greater set of Greeks and also
#' in the jump diffusion model
#' @seealso [Greeks_UI] for an interactive visualization
#'
#' @import "stats"
#' @import "Rcpp"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset, can also be a
#' vector
#' @param exercise_price - strike price of the option, can also be a vector
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param volatility - volatility of the underlying asset
#' @param dividend_yield - dividend yield
#' @param payoff - the payoff function, either a string in ("call", "put"), or a
#' function
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "rho",
#' "vega")
#' @param steps - the number of integration steps
#' @param paths - the number of simulated paths
#' @param seed - the seed of the random number generator
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples BS_Malliavin_Asian_Greeks(initial_price = 110, exercise_price = 100,
#' r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
#' greek = c("fair_value", "delta", "rho"), payoff = "put")
#'

BS_Malliavin_Asian_Greeks <- function(
    initial_price = 100,
    exercise_price = 100,
    r = 0,
    time_to_maturity = 1,
    volatility = 0.3,
    dividend_yield = 0,
    payoff = "call",
    greek = c("fair_value", "delta", "vega", "rho"),
    steps = round(time_to_maturity*252),
    paths = 1000,
    seed = 1) {

  ## cache stores value in order to compute them just once
  cache <- list()

  ## Finds the vectorized parameter, stores its name in `param` and its values
  ## in vectorized_param

  params <- c("initial_price", "exercise_price", "r", "time_to_maturity",
              "volatility", "dividend_yield")

  param <- params[1]
  vectorized_param <- get(param)

  for (p in params) {
    if (length(get(p)) >= 2) {
      vectorized_param <- get(p)
      param <- p
      break
    }
  }

  ## dt is the step size

  dt <- time_to_maturity/steps

  result <-
    matrix(ncol = length(greek),
           nrow = length(vectorized_param),
           dimnames = list(NULL, greek)) * NA

  ## the payoff function ##

  if (payoff == "call") {
    payoff_function <- function(x, exercise_price) {
      return(pmax(0, x - exercise_price))
    }
  } else if (payoff == "put") {
    payoff_function <- function(x, exercise_price) {
      return(pmax(0, exercise_price - x))
    }
  }

  ## the seed is set

  if (!is.na(seed)) {
    dqset.seed(seed)
  }

  gaussian_random_numbers <- dqrnorm(n = paths*steps, sd = sqrt(dt))
  W <- make_BM(gaussian_random_numbers, paths = paths, steps = steps)

  log_X <- calc_log_X(W, dt, volatility, r - dividend_yield)

  X <- exp(log_X)

  W_T <- W[, steps + 1]

  X_T <- X[, steps + 1]

  if ("vega" %in% greek) {
    XW <- calc_XW(X, W, steps, paths, dt)
    tXW <- calc_tXW(X, W, steps, paths, dt)
  }

  if ("vega" %in% greek) {
    I_W <- calc_I(W, steps, dt)
  }

  rm(W)

  ### the calculation of I_{(n)}, the integral (1/time_to_maturity) *
  ## (1/T) \int_0^T t^n X_t dt

  I_0 <- calc_I(X, steps, dt)

  if (length(intersect(
    greek,
    c("delta", "vega")))) {
    I_1 <- calc_I_1(X, steps, dt)
    I_2 <- calc_I_2(X, steps, dt)
  }

  cache$`exp(calc_I(log_X, steps, dt) / time_to_maturity)` <-
    exp(calc_I(log_X, steps, dt) / time_to_maturity)

  if("delta" %in% greek) {
    cache$`(1/(volatility) * (-volatility + I_0/I_1*W_T + volatility*I_0*I_2/(I_1^2)))` <-
      (1/(volatility) *
         (-volatility + I_0/I_1*W_T + volatility*I_0*I_2/(I_1^2)))
  }

  if ("vega" %in% greek) {
    cache$`vega_weight` <-
      ((1 / volatility) *
         ( -(1 + volatility * W_T) + (W_T * XW - volatility * tXW) / I_1 +
           (volatility * XW * I_2) / I_1^2))
  }

  for (i in 1:length(vectorized_param)) {

    assign(param, vectorized_param[i])

    I_0_geom <- initial_price * cache$`exp(calc_I(log_X, steps, dt) / time_to_maturity)`

    cache$`payoff_function(I_0_geom, exercise_price)` <-
      payoff_function(I_0_geom, exercise_price)

    geom_asian_greeks_exact <-
      BS_Geometric_Asian_Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        greek = greek
      )

    E_paths <- function(weight) {
      return(
        exp(-(r - dividend_yield) * time_to_maturity) *
          payoff_function(initial_price * I_0/time_to_maturity, exercise_price) *
               weight)
    }

    if("fair_value" %in% greek || "rho" %in% greek) {

      geom_fair_value <-
        exp(-(r - dividend_yield)*time_to_maturity) *
        cache$`payoff_function(I_0_geom, exercise_price)`

    }

    if ("fair_value" %in% greek) {

      fair_value <- E_paths(1)

      cont_fair_value <- geom_fair_value - geom_asian_greeks_exact["fair_value"]

      linear_model <- lm(fair_value ~ cont_fair_value)

      result[i, "fair_value"] <- linear_model$coefficients["(Intercept)"]

    }

    if("delta" %in% greek || "rho" %in% greek) {

      delta_geom_malliavin <-
        2*exp(-r*time_to_maturity)/(initial_price*volatility*time_to_maturity) *
        (cache$`payoff_function(I_0_geom, exercise_price)` * W_T)

    }

    if ("delta" %in% greek) {

      delta_malliavin <-
        (cache$`(1/(volatility) * (-volatility + I_0/I_1*W_T + volatility*I_0*I_2/(I_1^2)))` /
           initial_price) %>%
        E_paths()

      cont_delta_malliavin <-
        delta_geom_malliavin - geom_asian_greeks_exact["delta"]

      linear_model <- lm(delta_malliavin ~ cont_delta_malliavin)

      result[i, "delta"] <- linear_model$coefficients["(Intercept)"]

    }

    if ("rho" %in% greek) {
      rho_malliavin <-
        (W_T/volatility - time_to_maturity) %>%
        E_paths()

      rho_geom_malliavin <-
        -time_to_maturity * geom_fair_value +
        (initial_price * time_to_maturity)/2 * delta_geom_malliavin

      cont_rho_malliavin <-
        rho_geom_malliavin - geom_asian_greeks_exact["rho"]

      linear_model <- lm(rho_malliavin ~ cont_rho_malliavin)

      result[i, "rho"] <- linear_model$coefficients["(Intercept)"]

    }

    if ("vega" %in% greek) {

      vega_geom_malliavin <-
        exp(-r*time_to_maturity) *
        payoff_function(I_0_geom, exercise_price) *
        (2/(volatility * time_to_maturity**2) * W_T * I_W - 1/volatility - W_T)

      cont_vega_malliavin <-
        vega_geom_malliavin - geom_asian_greeks_exact["vega"]

      vega_malliavin <-
        cache$`vega_weight` %>%
        E_paths()

      linear_model <- lm(vega_malliavin ~ cont_vega_malliavin)

      result[i, "vega"] <- linear_model$coefficients["(Intercept)"]

    }

  }

  return(drop(result))

}

#' @title
#' Computes the implied volatility for various options via Newton's method
#'
#' @description
#' If the value of an option, and other (model)parameters like the risk-free
#' interest rate, the time-to-maturity, and the dividend yield are known, the
#' assumed volatility of the underlying asset, the *implied volatility* can be
#' inferred.
#' See Hull (2022).
#'
#' @references
#' Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition).
#' Pearson
#'
#' @export
#'
#' @seealso [BS_Implied_Volatility] for the special case
#' option_type = "European" and payoff in c("call", "put")
#'
#' @param option_price - current price of the option
#' @param initial_price - initial price of the underlying asset
#' @param exercise_price - strike price of the option
#' @param r - risk-free interest rate
#' @param time_to_maturity - time to maturity in years
#' @param dividend_yield - dividend yield
#' @param model - the model to be chosen
#' @param option_type in c("European", "American", "Geometric Asian", "Asian",
#' "Digital") - the type of option to be considered
#' @param payoff - in c("call", "put")
#' @param max_iter maximal number of iterations of the approximation
#' @param start_volatility initial guess
#' @param precision precision of the computation
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples Implied_Volatility(15, r = 0.05, option_type = "Asian",
#' payoff = "call")

Implied_Volatility <-
  function(
    option_price,
    initial_price = 100,
    exercise_price = 100,
    r = 0,
    time_to_maturity = 1,
    dividend_yield = 0,
    model = "Black_Scholes",
    option_type = "European",
    payoff = "call",
    start_volatility = 0.3,
    precision = 1e-6,
    max_iter = 30) {

    ## first the simplest European option case

    if (option_type == "European" && payoff %in% c("put", "call")) {
      return(BS_Implied_Volatility(
        option_price = option_price,
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        dividend_yield = dividend_yield,
        payoff = payoff,
        start_volatility = start_volatility))
    }

    ## Now the other cases

    # Start volatility given by the European case

    # We compute the option price with zero volatility to check if
    # BS_Implied_Volatility can deliver a viable starting value

    if(option_type == "Asian") {
      option_type_start_price <- "Geometric Asian"
    } else {
      option_type_start_price <- "European"
    }

    option_price_zero_vol <-
      Greeks(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = 1e-12,
        dividend_yield = dividend_yield,
        option_type = option_type_start_price,
        payoff = payoff,
        greek = "fair_value"
      )

    if(option_price > option_price_zero_vol) {

    start_volatility <-
      Implied_Volatility(
        option_price = option_price,
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        dividend_yield = dividend_yield,
        option_type = option_type_start_price,
        payoff = payoff,
        start_volatility = start_volatility)

    } else {
      start_volatility = 0.3
    }

    fair_value_and_vega_function <-
      function(volatility) {

        Greeks(initial_price = initial_price,
               exercise_price = exercise_price,
               r = r,
               time_to_maturity = time_to_maturity,
               volatility = volatility,
               dividend_yield = dividend_yield,
               model = model,
               option_type = option_type,
               payoff = payoff,
               greek = c("fair_value", "vega"))
      }

    volatility <- start_volatility

    ## check if option price can be obtained, we check for two volatility
    ## values, since volatility = 1e-9 gives NA for some option types

    option_price_0 <-
      min(
        Greeks(initial_price = initial_price,
               exercise_price = exercise_price,
               r = r,
               time_to_maturity = time_to_maturity,
               volatility = 1e-9,
               dividend_yield = dividend_yield,
               model = model,
               option_type = option_type,
               payoff = payoff,
               greek = "fair_value"),
        Greeks(initial_price = initial_price,
               exercise_price = exercise_price,
               r = r,
               time_to_maturity = time_to_maturity,
               volatility = 0.003,
               dividend_yield = dividend_yield,
               model = model,
               option_type = option_type,
               payoff = payoff,
               greek = "fair_value"),
        na.rm = TRUE
      )


    if (option_price_0 > option_price) {
      stop("Option price is too low. Implied volatility is not defined.")
    }

    iter <- 0

    while(TRUE) {

      iter <- iter + 1

      if (iter > max_iter) {
        stop("Computation not succesful. Try with lower precision, larger number of Iterations (max_iter) or larger number of paths")
      }

      fair_value_and_vega <- fair_value_and_vega_function(volatility)

      if (any(is.na(fair_value_and_vega))) {
        stop("Numerical error. Option price possibly too high.")
      }

      fair_value <- fair_value_and_vega["fair_value"]

      vega <- fair_value_and_vega["vega"]

      if (abs(fair_value - option_price) < precision) {
        return(volatility)
      }

      volatility <-
        max(1e-6, volatility - ((fair_value - option_price) / vega))

    }

  }

#' @title
#' Opens a shiny app to interactively visualize option prices and Greeks.
#'
#' @description
#' Opens a shiny app to interactively visualize option prices and Greeks.
#' This works for European Options (see [BS_European_Greeks]), American
#' Options (see [Binomial_American_Greeks]), Geometric Asian Options (see
#' [BS_Geometric_Asian_Greeks]), as well as Asian options (see
#' [BS_Malliavin_Asian_Greeks]).
#' For performance reasons, just the Black-Scholes model is possible, and for
#' some cases, the set of Greeks is limited.
#' On the y-Axis, the option value resp. the value of the greeks are displayed,
#' for the x-axis, several parameters like `initial_price` or `time_to_maturity`
#' are possible.
#'
#' \if{latex}{
#' \figure{GreeksUI.png}{The interface of the functions}
#' }
#'
#' @export
#'
#' @import "shiny"
#' @import "ggplot2"
#' @import "tibble"
#' @import "tidyr"
#' @importFrom "plotly" "renderPlotly" "ggplotly" "plotlyOutput"
#'

Greeks_UI <- function() {

  greeks_list <-
    list(
      "Fair Value" = "fair_value",
      "Delta" = "delta",
      "Vega" = "vega",
      "Theta" = "theta",
      "Rho" = "rho",
      "Epsilon" = "epsilon",
      "Lambda" = "lambda",
      "Gamma" = "gamma",
      "Vanna" = "vanna",
      "Charm" = "charm",
      "Vomma" = "vomma",
      "Veta" = "veta",
      "Speed" = "speed")

  # TODO: Maybe this could be done in a more elegant way
  greeks_list_reversed <- names(greeks_list)
  names(greeks_list_reversed) <- as.vector(greeks_list)

  params_list <-
    list(
      "Initial Price" = "initial_price",
      "Exercise Price" = "exercise_price",
      "Riskless Interest Rate" = "r",
      "Time to Maturity" = "time_to_maturity",
      "Volatility" = "volatility",
      "Dividend Yield" = "dividend_yield")

  ############################################################################
  ####### first row of selection panels with 'X-Axis' and 'Option Type' ######
  ############################################################################

  ui <- fluidPage(
    fluidRow(
      # x-axis
      conditionalPanel(
        condition = ("input.option_type == 'Asian'"),
        column(
          width = 6,
          selectInput(
            inputId = "x_axis_asian",
            label = "X-axis",
            choices = c("Initial Price", "Exercise Price"),
            selected = "initial_price",
            multiple = FALSE)
        )
      ), # conditionalPanel
      conditionalPanel(
        condition = ("input.option_type != 'Asian'"),
        column(
          width = 6,
          selectInput(
            inputId = "x_axis",
            label = "X-axis",
            choices = names(params_list),
            selected = "initial_price",
            multiple = FALSE)
        )
      ), # conditionalPanel
      # option type
      column(
        width = 6,
        selectInput(
          inputId = "option_type",
          label = "Option Type",
          choices = list("European", "American", "Geometric Asian", "Asian"),
          selected = "European",
          multiple = FALSE)
      )
    ), # fluidRow

    ############################################################################
    ######### second row of selection panels with 'Greek' and 'Payoff' #########
    ############################################################################

    fluidRow(
      # greek for European Options
      conditionalPanel(
        condition = ("input.option_type == 'European'"),
        column(
          width = 6,
          selectInput(
            inputId = "greek_european",
            label = "Greek",
            choices = names(greeks_list),
            selected = c("Fair Value", "Delta"),
            multiple = TRUE)
        )
      ), # conditionalPanel
      # greek for American Options
      conditionalPanel(
        condition = ("input.option_type == 'American'"),
        column(
          width = 6,
          selectInput(
            inputId = "greek_american",
            label = "Greek",
            choices = c("Fair Value", "Delta", "Vega", "Theta", "Rho",
                        "Epsilon", "Gamma"),
            selected = c("Fair Value", "Delta"),
            multiple = TRUE)
        )
      ), # conditionalPanel
      # greek for Geometric Asian Options
      conditionalPanel(
        condition = ("input.option_type == 'Geometric Asian'"),
        column(
          width = 6,
          selectInput(
            inputId = "greek_geometric_asian",
            label = "Greek",
            choices = c("Fair Value", "Delta", "Vega", "Theta", "Rho",
                        "Epsilon", "Gamma"),
            selected = c("Fair Value", "Delta"),
            multiple = TRUE)
        )
      ), # conditionalPanel
      # greek for Geometric Asian Options
      conditionalPanel(
        condition = ("input.option_type == 'Asian'"),
        column(
          width = 6,
          selectInput(
            inputId = "greek_asian",
            label = "Greek",
            choices = c("Fair Value", "Delta", "Rho", "Vega"),
            selected = c("Fair Value", "Delta"),
            multiple = TRUE)
        )
      ), # conditionalPanel
      # payoff
      conditionalPanel(
        condition = ("input.option_type == 'European'"),
        column(
          width = 6,
          selectInput(
            inputId = "payoff_european",
            label = "Payoff",
            choices = list(
              "Call" = "call",
              "Put" = "put",
              "Cash or nothing Call" = "cash_or_nothing_call",
              "Cash or nothing Put" = "cash_or_nothing_put",
              "Asset or nothing Call" = "asset_or_nothing_call",
              "Asset or nothing Put" =  "asset_or_nothing_put"),
            selected = "call",
            multiple = FALSE)
        )
      ), # conditionalPanel
      # payoff
      conditionalPanel(
        condition = ("input.option_type != 'European'"),
        column(
          width = 6,
          selectInput(
            inputId = "payoff",
            label = "Payoff",
            choices = list(
              "Call" = "call",
              "Put" = "put"),
            selected = "call",
            multiple = FALSE)
        )
      ) # conditionalPanel
    ), # fluidRow

    ############################################################################
    ### first row of slider panels with 'Initial Price' and 'Exercise Price' ###
    ############################################################################

    fluidRow(

      # Initial Price
      conditionalPanel(
        condition = ("((input.option_type != 'Asian') && (input.x_axis != 'Initial Price'))"),
        column(
          width = 6,
          sliderInput(
            inputId = "initial_price",
            label = "Initial Price",
            min = 0,
            max = 200,
            value = 100
          )
        )
      ), # conditionalPanel
      # Initial Price
      conditionalPanel(
        condition = ("input.x_axis == 'Initial Price'"),
        column(
          width = 6,
          sliderInput(
            inputId = "initial_price_2",
            label = "Initial Price",
            min = 0,
            max = 200,
            value = c(1, 200)
          )
        )
      ), # condionalPanel

      # Exercise Price
      conditionalPanel(
        condition = ("input.x_axis != 'Exercise Price'"),
        column(
          width = 6,
          sliderInput(
            inputId = "exercise_price",
            label = "Exercise Price",
            min = 0,
            max = 200,
            value = 100
          )
        )
      ), # conditionalPanel
      # Exercise Price
      conditionalPanel(
        condition = ("input.x_axis == 'Exercise Price'"),
        column(
          width = 6,
          sliderInput(
            inputId = "exercise_price_2",
            label = "Exercise Price",
            min = 1,
            max = 200,
            value = c(1, 200)
          )
        )
      ), # conditionalPanel
    ), # fluidRow

    ############################################################################
    ######### second row of slider panels with 'Riskless Interest Rate' ########
    ########################## and 'Time to Maturity' ##########################
    ############################################################################

    fluidRow(

      # Riskless Interest Rate
      conditionalPanel(
        condition = ("input.x_axis != 'Riskless Interest Rate'"),
        column(
          width = 6,
          sliderInput(
            inputId = "r",
            label = "Riskless Interest Rate",
            min = -0.1,
            max = 1,
            value = 0
          )
        )
      ), # conditionalPanel
      # Riskless Interest Rate
      conditionalPanel(
        condition = ("input.x_axis == 'Riskless Interest Rate'"),
        column(
          width = 6,
          sliderInput(
            inputId = "r_2",
            label = "Riskless Interest Rate",
            min = -0.1,
            max = 1,
            value = c(-0.1, 1)
          )
        )
      ), # conditionalPanel

      # Time to Maturity
      conditionalPanel(
        condition = ("input.x_axis != 'Time to Maturity'"),
        column(
          width = 6,
          sliderInput(
            inputId = "time_to_maturity",
            label = "Time to Maturity",
            min = 0,
            max = 20,
            value = 1,
            step = 0.1
          )
        )
      ), # conditionalPanel
      # Time to Maturity
      conditionalPanel(
        condition = ("input.x_axis == 'Time to Maturity'"),
        column(
          width = 6,
          sliderInput(
            inputId = "time_to_maturity_2",
            label = "Time to Maturity",
            min = 0,
            max = 20,
            value = c(0, 20),
            step = 0.1
          )
        )
      ) # conditionalPanel
    ), # fluidRow

    ############################################################################
    ##### third row of slider panels with 'Volatility' and 'Dividend Yield #####
    ############################################################################

    fluidRow(

      # Volatility
      conditionalPanel(
        condition = ("input.x_axis != 'Volatility'"),
        column(
          width = 6,
          sliderInput(
            inputId = "volatility",
            label = "Volatility",
            min = 0,
            max = 1,
            value = 0.3
          )
        )
      ), # conditionalPanel
      # Volatility
      conditionalPanel(
        condition = ("input.x_axis == 'Volatility'"),
        column(
          width = 6,
          sliderInput(
            inputId = "volatility_2",
            label = "Volatility",
            min = 0.01,
            max = 1,
            value = c(0.01, 1)
          )
        )
      ), # conditionalPanel

      # Dividend Yield
      conditionalPanel(
        condition = ("input.x_axis != 'Dividend Yield'"),
        column(
          width = 6,
          sliderInput(
            inputId = "dividend_yield",
            label = "Dividend Yield",
            min = 0,
            max = 1,
            value = 0
          )
        )
      ), # conditionalPanel
      # Dividend Yield
      conditionalPanel(
        condition = ("input.x_axis == 'Dividend Yield'"),
        column(
          width = 6,
          sliderInput(
            inputId = "dividend_yield_2",
            label = "Dividend Yield",
            min = 0,
            max = 1,
            value = c(0, 1)
          )
        )
      ) # conditionalPanel
    ), # fluidRow

    plotlyOutput(outputId = "plot")
  )

  server <- function(input, output) {

    output$plot <- renderPlotly(
      {

        ## The parameters for the computation of the Greeks are extracted from
        ## input

        initial_price <- input$initial_price
        exercise_price <- input$exercise_price
        r <- input$r
        time_to_maturity <- input$time_to_maturity
        volatility <- input$volatility
        dividend_yield <- input$dividend_yield

        payoff <-
          ifelse(input$option_type == "European",
                 payoff <- input$payoff_european,
                 payoff <- input$payoff)

        if (input$option_type == "European") {
          greek <- input$greek_european
        } else if (input$option_type == "American") {
          greek <- input$greek_american
        } else if (input$option_type == "Geometric Asian") {
          greek <- input$greek_geometric_asian
        } else if (input$option_type == "Asian") {
          greek <- input$greek_asian
        }

        greek <- greeks_list[greek] %>%
          unlist() %>%
          unname()

        if (input$option_type == "Asian") {
          x_bounds <-
            input[[eval(paste(params_list[[input$x_axis_asian]], "_2", sep = ""))]]
        } else {
          x_bounds <-
            input[[eval(paste(params_list[[input$x_axis]], "_2", sep = ""))]]
        }

        x_from <- x_bounds[1]
        x_to <- x_bounds[2]

        step_size <- (x_to - x_from)/200

        precision <- max(ceiling(-log(step_size, base = 10)), 4)

        step_size <- round(step_size, precision)

        x <- seq(x_from, x_to, by = step_size)

        if (input$option_type == "Asian") {
          assign(params_list[[input$x_axis_asian]], x)
        } else {
          assign(params_list[[input$x_axis]], x)
        }

        if(input$option_type == "Asian") {

          Option_price <-
            BS_Malliavin_Asian_Greeks(
              initial_price = initial_price,
              exercise_price = exercise_price,
              r = r,
              time_to_maturity = time_to_maturity,
              volatility = volatility,
              dividend_yield = dividend_yield,
              payoff = payoff,
              greek = greek
            ) %>% as_tibble() %>%
            add_column(x = x) %>%
            pivot_longer(cols = -x,
                         names_to = "Greek",
                         values_to = "Value")

          # The entries of the Greeks column are change to a more human readable
          # format
          Option_price$Greek <- greeks_list_reversed[Option_price$Greek]

        } else {

          ## FUN is the function applied to sapply that compute the Greeks

          FUN <- function(x) {
            assign(params_list[[input$x_axis]], x)

            Greeks(
              initial_price = initial_price,
              exercise_price = exercise_price,
              r = r,
              time_to_maturity = time_to_maturity,
              volatility = volatility,
              dividend_yield = dividend_yield,
              option_type = input$option_type,
              payoff = payoff,
              greek = greek,
              steps = 100) %>%
              round(4)
          }

          Option_price_list <-
            lapply(
              X = matrix(x),
              FUN = FUN
            )

          Option_price <-
            matrix(nrow = length(x), ncol = length(greek))

          for(row in 1:length(x)) {
            Option_price[row, ] <- round(Option_price_list[[row]], 4)
          }

          # The entries of the Greeks column are change to a more human readable
          # format
          colnames(Option_price) <- greeks_list_reversed[greek]

          Option_price <-
            Option_price %>%
            as_tibble() %>%
            add_column(x) %>%
            pivot_longer(cols = -x,
                         names_to = "Greek",
                         values_to = "Value")

        }

        colnames(Option_price) <-
          replace(colnames(Option_price), colnames(Option_price) == "x", input$x_axis)

        plot <-
          Option_price %>%
          ggplot() +
          geom_line(mapping = aes(x = .data[[input$x_axis]],
                                  y = .data$Value,
                                  color = .data$Greek)) +
          theme_minimal() +
          xlab(input$x_axis) +
          ggtitle(paste("Prices and Sensitivites of", input$option_type, "Options"))

        ggplotly(plot)

      }
    )
  }

  shinyApp(ui = ui, server = server, options = list(height = 1000))

}

#' @title
#' Computes the Greeks of an American call- or put-option with the Binomial
#' options pricing model
#'
#' @description
#' In contract to European Options, American options can be executed at any time
#' until the expiration date.
#' For more details on the definition of Greeks in general see [Greeks].
#' This functions computes Greeks of American put- and call options in the
#' binomial option pricing model (see (Hull, 2022)).
#'
#' @export
#'
#' @seealso [Greeks_UI] for an interactive visualization
#'
#' @import "stats"
#' @import "Rcpp"
#' @importFrom "dqrng" "dqrnorm" "dqset.seed"
#'
#' @param initial_price - initial price of the underlying asset.
#' @param exercise_price - strike price of the option.
#' @param r - risk-free interest rate.
#' @param time_to_maturity - time to maturity.
#' @param volatility - volatility of the underlying asset.
#' @param dividend_yield - dividend yield.
#' @param payoff - the payoff function, a string in ("call", "put").
#' @param greek - Greeks to be calculated in c("fair_value", "delta", "vega",
#' "theta", "rho", "epsilon", "gamma")
#' @param steps - the number of integration steps.
#' @param eps - the step size for the finite difference method to calculate
#' theta, vega, rho and epsilon
#'
#' @useDynLib greeks, .registration=TRUE
#'
#' @return Named vector containing the values of the Greeks specified in the
#' parameter \code{greek}.
#'
#' @examples Binomial_American_Greeks(initial_price = 100, exercise_price = 100,
#' r = 0, time_to_maturity = 1, volatility = 0.3, dividend_yield = 0,
#' payoff = "call", greek = c("fair_value", "delta", "vega", "theta", "rho",
#' "epsilon", "gamma"), steps = 20)
#'
#'@references
#' Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition). Pearson
#'

Binomial_American_Greeks <-
  function(initial_price = 100,
           exercise_price = 100,
           r = 0,
           time_to_maturity = 1,
           volatility = 0.3,
           dividend_yield = 0,
           payoff = "call",
           greek = c("fair_value", "delta", "vega", "theta", "rho", "epsilon",
                     "gamma"),
           steps = 1000,
           eps = 1/100000) {

    if (length(payoff) != 1 || !(payoff %in% c("call", "put"))) {
      stop("payoff must be either 'call' or 'put'.", call. = FALSE)
    }

    ## Definition of a helper-function of make better use of
    ## Binomial_American_Greeks.cpp

    compute_fair_value <- function(...) {

      if (length(list(...)) >= 1) {
        for (i in 1:length(list(...))) {
          param_name <- names(list(...)[i])
          param_value <- list(...)[[i]]
          assign(x = param_name, value = param_value)
        }
      }

      v <- Binomial_American_Greeks_cpp(
        initial_price = initial_price,
        exercise_price = exercise_price,
        r = r,
        time_to_maturity = time_to_maturity,
        volatility = volatility,
        dividend_yield = dividend_yield,
        payoff = payoff,
        steps = steps)

      return(
        v["fair_value"] +
          BS_European_Greeks(
            initial_price = initial_price,
            exercise_price = exercise_price,
            r = r,
            time_to_maturity = time_to_maturity,
            volatility = volatility,
            dividend_yield = dividend_yield,
            payoff = payoff,
            greek = "fair_value") -
          v["european_fair_value"]
      )
    }

    ## the actual function definition
    result <- numeric(length(greek)) * NA

    names(result) <- greek

    if ("fair_value" %in% greek) {
      result["fair_value"] <- compute_fair_value()
    }

    if ("delta" %in% greek) {
      v_up <- compute_fair_value(initial_price = initial_price + eps)
      v_down <- compute_fair_value(initial_price = initial_price - eps)
      result["delta"] <- (v_up - v_down) / (2*eps)

    }

    if ("gamma" %in% greek) {
      eps_gamma <- initial_price/50

      v_up <- compute_fair_value(initial_price = initial_price + eps_gamma)
      v <- compute_fair_value(initial_price = initial_price)
      v_down <- compute_fair_value(initial_price = initial_price - eps_gamma)

      result["gamma"] <-
        (v_up - 2*v + v_down) / (eps_gamma**2)

    }

    if ("vega" %in% greek) {
      v_up <- compute_fair_value(volatility = volatility + eps)
      v_down <- compute_fair_value(volatility = volatility - eps)
      result["vega"] <- (v_up - v_down) / (2*eps)
    }

    if ("theta" %in% greek) {
      v_up <- compute_fair_value(time_to_maturity = time_to_maturity + eps)
      v_down <- compute_fair_value(time_to_maturity = time_to_maturity - eps)
      result["theta"] <- - (v_up - v_down) / (2*eps)
    }

    if ("rho" %in% greek) {
      v_up <- compute_fair_value(r = r + eps)
      v_down <- compute_fair_value(r = r - eps)
      result["rho"] <- (v_up - v_down) / (2*eps)
    }

    if ("epsilon" %in% greek) {
      v_up <- compute_fair_value(dividend_yield = dividend_yield + eps)
      v_down <- compute_fair_value(dividend_yield = dividend_yield - eps)
      result["epsilon"] <- (v_up - v_down) / (2*eps)
    }

    return(result)

  }

---
title: "Implied volatility"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Implied volatility}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(greeks)
```

Implied volatility is the volatility parameter that makes a model price match an
observed option price. In `greeks`, `BS_Implied_Volatility()` handles European
Black-Scholes calls and puts directly. `Implied_Volatility()` provides a wrapper
for other option types by repeatedly evaluating the corresponding pricing
function and vega.

The test suite checks implied volatility by a round trip:

1. choose a volatility;
2. compute an option price from that volatility;
3. infer implied volatility from the option price;
4. price the same option again with the inferred volatility.

## European Black-Scholes implied volatility

```{r}
true_volatility <- 0.28

option_price <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  volatility = true_volatility,
  payoff = "call",
  greek = "fair_value"
)

implied_volatility <- BS_Implied_Volatility(
  option_price = option_price,
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  payoff = "call",
  start_volatility = 0.2
)

round(
  c(
    option_price = option_price,
    true_volatility = true_volatility,
    implied_volatility = implied_volatility
  ),
  6
)
```

The reconstructed price should match the original option price up to numerical
precision.

```{r}
reconstructed_price <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1.25,
  dividend_yield = 0.01,
  volatility = implied_volatility,
  payoff = "call",
  greek = "fair_value"
)

round(
  c(
    original_price = option_price,
    reconstructed_price = reconstructed_price,
    absolute_error = abs(option_price - reconstructed_price)
  ),
  10
)
```

## Implied volatility for other option types

For non-European options, use `Implied_Volatility()`. The wrapper uses the same
pricing entry point as `Greeks()`.

```{r}
geometric_true_volatility <- 0.35

geometric_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = geometric_true_volatility,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

geometric_implied_volatility <- Implied_Volatility(
  option_price = geometric_price,
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  option_type = "Geometric Asian",
  payoff = "put"
)

geometric_reconstructed_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = geometric_implied_volatility,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

round(
  c(
    original_price = geometric_price,
    true_volatility = geometric_true_volatility,
    implied_volatility = geometric_implied_volatility,
    reconstructed_price = geometric_reconstructed_price,
    absolute_error = abs(geometric_price - geometric_reconstructed_price)
  ),
  6
)
```

## Prices below the zero-volatility value

An implied volatility is not defined when the requested option price is below
the lowest value attainable by the pricing model. A practical first check is to
compare the observed price with a near-zero-volatility model value.

```{r}
near_zero_volatility_price <- Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 1e-12,
  option_type = "Geometric Asian",
  payoff = "put",
  greek = "fair_value"
)

round(near_zero_volatility_price, 6)
```

If the observed option price is lower than this value, the implied volatility
calculation should be treated as infeasible rather than as a numerical tuning
problem.

## Reference

The concept of implied volatility and the Black-Scholes model are standard
option pricing material; see Hull (2022).

---
title: "Asian options and Greeks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Asian options and Greeks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(greeks)
```

Asian options are path-dependent: their payoff depends on an average of the
underlying asset price over time. The package supports two related cases:

- geometric Asian options, where the geometric average is used and exact
  Black-Scholes formulas are available for call and put payoffs;
- arithmetic Asian options, where Malliavin Monte Carlo methods are used.

The package tests compare exact formulas, Monte Carlo estimates, and finite
differences. This vignette turns those checks into small reproducible examples.

## Geometric Asian options

`BS_Geometric_Asian_Greeks()` computes exact Black-Scholes values for geometric
Asian calls and puts.

```{r}
geometric_exact <- BS_Geometric_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma")
)

round(geometric_exact, 4)
```

The tests check the formulas by comparing each Greek with a finite difference.
Here is that idea for vega, the derivative with respect to volatility.

```{r}
geometric_price_at <- function(volatility) {
  BS_Geometric_Asian_Greeks(
    initial_price = 110,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1.5,
    dividend_yield = 0.01,
    volatility = volatility,
    payoff = "call",
    greek = "fair_value"
  )
}

step_size <- 1e-4
finite_difference_vega <-
  (geometric_price_at(0.25 + step_size) -
     geometric_price_at(0.25 - step_size)) /
  (2 * step_size)

round(
  c(
    exact_vega = geometric_exact["vega"],
    finite_difference_vega = finite_difference_vega,
    absolute_error = abs(geometric_exact["vega"] - finite_difference_vega)
  ),
  8
)
```

`Malliavin_Geometric_Asian_Greeks()` can estimate the same quantities by
simulation. A moderate number of paths is enough for a vignette example; for
production work, increase `paths` and check stability.

```{r}
greeks_to_compare <- c("fair_value", "delta", "rho", "vega")

geometric_monte_carlo <- Malliavin_Geometric_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare,
  paths = 20000,
  steps = 24,
  seed = 42,
  antithetic = TRUE
)

round(
  rbind(
    exact = geometric_exact[greeks_to_compare],
    malliavin_monte_carlo = geometric_monte_carlo
  ),
  4
)
```

## Arithmetic Asian options

For arithmetic Asian options the package uses Malliavin Monte Carlo estimators.
`BS_Malliavin_Asian_Greeks()` is optimized for the Black-Scholes model and the
most common Greeks.

```{r}
arithmetic_asian <- BS_Malliavin_Asian_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0.01,
  volatility = 0.25,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega"),
  paths = 10000,
  steps = 24,
  seed = 42
)

round(arithmetic_asian, 4)
```

The generic `Greeks()` wrapper dispatches to the Asian implementation when
`option_type = "Asian"`.

```{r}
round(
  Greeks(
    initial_price = 110,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1.5,
    dividend_yield = 0.01,
    volatility = 0.25,
    payoff = "call",
    option_type = "Asian",
    greek = c("fair_value", "delta", "rho", "vega")
  ),
  4
)
```

## Vectorized inputs and custom payoffs

The Malliavin Asian functions can vary one scalar parameter over a vector. This
is useful for plotting how values change with the current underlying price.

```{r}
price_grid <- Malliavin_Geometric_Asian_Greeks(
  initial_price = c(95, 100, 105),
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "put",
  greek = c("fair_value", "delta"),
  paths = 5000,
  steps = 12,
  seed = 42
)

round(price_grid, 4)
```

Custom payoff functions are also accepted. The payoff function receives the
averaged underlying value and the exercise price.

```{r}
digital_call <- function(x, exercise_price) {
  ifelse(x >= exercise_price, 1, 0)
}

invisible(capture.output(
  custom_digital <- Malliavin_Geometric_Asian_Greeks(
    initial_price = 100,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 1,
    volatility = 0.25,
    payoff = digital_call,
    greek = c("fair_value", "delta"),
    paths = 5000,
    steps = 12,
    seed = 42
  )
))

round(custom_digital, 4)
```

## References

Asian option pricing and Greeks are discussed in Hull (2022). The Malliavin
Monte Carlo estimators used here are described in Hudde and Rueschendorf (2023).

---
title: "greeks: Sensitivities of Prices of Financial Options and Implied Volatilities"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{greeks: Sensitivities of Prices of Financial Options and Implied Volatilities}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The package `greeks` provides functions to compute financial option prices and
sensitivities of financial option prices for European, American, Asian, and
Digital options in the Black Scholes model, and in more general jump diffusion
models.
Furthermore, based on the implementations of Vega, efficient functions for
calculating implied volatilities not just for European options, but also for
American, Asian, and digital options are provided.

Classical formulas are implemented for European options in the Black Scholes
Model. 
Furthermore, functions to calculate Malliavin Monte Carlo Greeks are given, as
presented e.g., in Hudde, A. & Rüschendorf, L. (2016). 
European and Asian Greeks for exponential Lévy processes
(https://arxiv.org/abs/1603.00920).
These functions work for classical payoff functions, as well as for any custom
square integrable function provided by the user.
Additionally, these calculations are not restricted to the Black Scholes model,
but work for more general Lévy Jump diffusion model, which is also customizable
by the user.

## Computing option prices and Greeks in the shiny app

The option prices and volatilities for European options can be displayed with
the interactive shiny app by calling `Greeks_UI`

<img src="GreeksUI.png" width="600" class="center">

## The function Greeks

Most of the options prices and Greeks can easily be calculated with the function
Greeks.

    # Load package

    library(greeks)

    # Option price and most common Greeks of a European call option on a share with
    # price 100 and volatility of 30%, where the exercise price is 120, the time to
    # maturity of 5 years, and the riskless interest rate of 1%.

    Greeks(initial_price = 100,
           exercise_price = 120,
           r = 0.01,
           time_to_maturity = 5,
           volatility = 0.30,
           payoff = "call")

    ##    fair_value         delta          vega         theta           rho 
    ##  21.577149923   0.554941778  88.358901748  -2.989937331 169.585139380 
    ##         gamma 
    ##   0.005890593

    # Option price and most common Greeks of an American put option on a share with
    # price 100 and volatility of 25%, where the exercise price is 100, the time to
    # maturity of 1 year, and the riskless interest rate of -0.5%.

    Greeks(initial_price = 100,
           exercise_price = 100,
           r = -0.005,
           time_to_maturity = 1,
           volatility = 0.30,
           payoff = "put",
           option_type = "American")

    ##  fair_value       delta        vega       theta         rho       gamma 
    ##  12.2027075  -0.4469782  39.5313017  -6.2141979 -56.9005269  -0.1275472
    
## Computing implied volatilities

The package `greeks` also provides a function to compute implied volatilities for a wide range of option types and payoff functions:

    # Implied volatility of an Asian call option with on an option price of 15, a
    # share price of 100, an exercise_price of 100, a risk-free interest rate of
    # 0.05 and a time to maturity of 1.
    
    Implied_Volatility(option_price = 15, initial_price = 100,
                 exercise_price = 100, r = 0.05, time_to_maturity = 1,
                 option_type = "Asian", payoff = "call")
    
    ## [1] 0.6330451

---
title: "American options and Greeks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{American options and Greeks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(greeks)
```

American options can be exercised at any time before maturity. The package
prices American calls and puts with a binomial tree via
`Binomial_American_Greeks()`. The fair value is computed by the tree, and the
Greeks are computed by finite differences around that fair value.

The tests verify the binomial implementation against an independent tree and
check finite-difference Greeks. The examples below show the same ideas on small,
readable inputs.

## Pricing an American option

```{r}
american_put <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = c("fair_value", "delta", "gamma", "vega", "theta", "rho"),
  steps = 200
)

round(american_put, 4)
```

The generic `Greeks()` wrapper can be used for the same option by setting
`option_type = "American"`.

```{r}
round(
  Greeks(
    initial_price = 100,
    exercise_price = 100,
    r = 0.03,
    time_to_maturity = 1,
    dividend_yield = 0,
    volatility = 0.3,
    payoff = "put",
    option_type = "American",
    greek = c("fair_value", "delta", "rho"),
    steps = 200
  ),
  4
)
```

## Early-exercise premium

For a put option, the American value can exceed the corresponding European
Black-Scholes value because early exercise may be beneficial.

```{r}
european_put_value <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "fair_value"
)

american_put_value <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "fair_value",
  steps = 400
)

round(
  c(
    european_put = european_put_value,
    american_put = american_put_value,
    early_exercise_premium = american_put_value - european_put_value
  ),
  4
)
```

## Choosing the number of tree steps

A larger number of tree steps generally gives a more accurate binomial
approximation, at the cost of more computation.

```{r}
steps <- c(25, 50, 100, 200, 400)

prices <- vapply(
  steps,
  function(n_steps) {
    Binomial_American_Greeks(
      initial_price = 100,
      exercise_price = 100,
      r = 0.03,
      time_to_maturity = 1,
      dividend_yield = 0,
      volatility = 0.3,
      payoff = "put",
      greek = "fair_value",
      steps = n_steps
    )
  },
  numeric(1)
)

data.frame(steps = steps, fair_value = round(prices, 4))
```

## A finite-difference delta

The implementation computes American Greeks by perturbing one model parameter at
a time. This reproduces the calculation for delta.

```{r}
fair_value_at <- function(initial_price) {
  Binomial_American_Greeks(
    initial_price = initial_price,
    exercise_price = 100,
    r = 0.03,
    time_to_maturity = 1,
    dividend_yield = 0,
    volatility = 0.3,
    payoff = "put",
    greek = "fair_value",
    steps = 200
  )
}

step_size <- 0.01
finite_difference_delta <-
  (fair_value_at(100 + step_size) - fair_value_at(100 - step_size)) /
  (2 * step_size)

reported_delta <- Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0.03,
  time_to_maturity = 1,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "put",
  greek = "delta",
  steps = 200,
  eps = step_size
)

round(
  c(
    reported_delta = reported_delta,
    finite_difference_delta = finite_difference_delta,
    absolute_error = abs(reported_delta - finite_difference_delta)
  ),
  8
)
```

## Reference

The binomial option pricing model is presented in Hull (2022).

---
title: "European options and Greeks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{European options and Greeks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(greeks)
```

European options can be exercised only at maturity. In the Black-Scholes model,
the package computes their prices and sensitivities with closed formulas through
`BS_European_Greeks()`. The generic `Greeks()` wrapper dispatches to the same
implementation when `option_type = "European"` and `model = "Black_Scholes"`.

The examples below are compact versions of the checks used in the test suite:
they compute a few Greeks directly, check one Greek with a finite difference,
and compare exact Black-Scholes values with the Malliavin Monte Carlo estimator.

## Exact Black-Scholes values

The `greek` argument can contain more than one quantity. The result is a named
numeric vector.

```{r}
european_put <- BS_European_Greeks(
  initial_price = 120,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 4.5,
  dividend_yield = 0.015,
  volatility = 0.22,
  payoff = "put",
  greek = c("fair_value", "delta", "gamma", "vega", "theta", "rho")
)

round(european_put, 4)
```

The same calculation can be written with the wrapper:

```{r}
round(
  Greeks(
    initial_price = 120,
    exercise_price = 100,
    r = 0.02,
    time_to_maturity = 4.5,
    dividend_yield = 0.015,
    volatility = 0.22,
    payoff = "put",
    greek = c("fair_value", "delta", "gamma")
  ),
  4
)
```

Digital payoffs are supported by the European Black-Scholes implementation. A
cash-or-nothing call pays one unit of cash at maturity if the option finishes in
the money.

```{r}
digital_call <- BS_European_Greeks(
  initial_price = 100,
  exercise_price = 105,
  r = 0.03,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "cash_or_nothing_call",
  greek = c("fair_value", "delta", "vega")
)

round(digital_call, 4)
```

## A finite-difference check

Delta is the derivative of the option value with respect to the initial price
of the underlying asset. The tests verify this over many random inputs. For a
single option, the same idea can be seen with a central finite difference.

```{r}
base_args <- list(
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1.5,
  dividend_yield = 0,
  volatility = 0.3,
  payoff = "call"
)

fair_value_at <- function(initial_price) {
  do.call(
    BS_European_Greeks,
    c(base_args, list(initial_price = initial_price, greek = "fair_value"))
  )
}

step_size <- 1e-4
finite_difference_delta <-
  (fair_value_at(100 + step_size) - fair_value_at(100 - step_size)) /
  (2 * step_size)

exact_delta <- do.call(
  BS_European_Greeks,
  c(base_args, list(initial_price = 100, greek = "delta"))
)

round(
  c(
    exact_delta = exact_delta,
    finite_difference_delta = finite_difference_delta,
    absolute_error = abs(exact_delta - finite_difference_delta)
  ),
  8
)
```

## Malliavin Monte Carlo comparison

`Malliavin_European_Greeks()` estimates the same Greeks by simulation. This is
less efficient than closed formulas for plain European options, but it is useful
as a bridge to the Malliavin methods used for path-dependent options.

```{r}
greeks_to_compare <- c("fair_value", "delta", "vega", "theta", "rho", "gamma")

exact <- BS_European_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare
)

monte_carlo <- Malliavin_European_Greeks(
  initial_price = 110,
  exercise_price = 100,
  r = 0.02,
  time_to_maturity = 1,
  volatility = 0.25,
  payoff = "call",
  greek = greeks_to_compare,
  paths = 50000,
  seed = 42,
  antithetic = TRUE
)

round(rbind(exact = exact, malliavin_monte_carlo = monte_carlo), 4)
```

## References

The closed-form European formulas are standard Black-Scholes results; see Hull
(2022). The Malliavin estimators are described in Hudde and Rueschendorf (2023).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Implied_Volatility.R
\name{Implied_Volatility}
\alias{Implied_Volatility}
\title{Computes the implied volatility for various options via Newton's method}
\usage{
Implied_Volatility(
  option_price,
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  dividend_yield = 0,
  model = "Black_Scholes",
  option_type = "European",
  payoff = "call",
  start_volatility = 0.3,
  precision = 1e-06,
  max_iter = 30
)
}
\arguments{
\item{option_price}{\itemize{
\item current price of the option
}}

\item{initial_price}{\itemize{
\item initial price of the underlying asset
}}

\item{exercise_price}{\itemize{
\item strike price of the option
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{model}{\itemize{
\item the model to be chosen
}}

\item{option_type}{in c("European", "American", "Geometric Asian", "Asian",
"Digital") - the type of option to be considered}

\item{payoff}{\itemize{
\item in c("call", "put")
}}

\item{start_volatility}{initial guess}

\item{precision}{precision of the computation}

\item{max_iter}{maximal number of iterations of the approximation}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
If the value of an option, and other (model)parameters like the risk-free
interest rate, the time-to-maturity, and the dividend yield are known, the
assumed volatility of the underlying asset, the \emph{implied volatility} can be
inferred.
See Hull (2022).
}
\examples{
Implied_Volatility(15, r = 0.05, option_type = "Asian",
payoff = "call")
}
\references{
Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition).
Pearson
}
\seealso{
\link{BS_Implied_Volatility} for the special case
option_type = "European" and payoff in c("call", "put")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Greeks.R
\name{Greeks}
\alias{Greeks}
\title{Computes the Greeks of various options in the Black Scholes model or
both in the Black Scholes model or a Jump Diffusion model in the case of Asian
Options, or in the Binomial options pricing model}
\usage{
Greeks(
  initial_price,
  exercise_price,
  r,
  time_to_maturity,
  volatility,
  dividend_yield = 0,
  model = "Black_Scholes",
  option_type = "European",
  payoff = "call",
  greek = c("fair_value", "delta", "vega", "theta", "rho", "gamma"),
  antithetic = TRUE,
  ...
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset
}}

\item{exercise_price}{\itemize{
\item strike price of the option
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{model}{\itemize{
\item the model to be chosen in ("black_scholes", "jump_diffusion")
}}

\item{option_type}{in c("European", "American", "Asian", "Geometric Asian",
"Digital", "Binomial) - the type of option to be considered}

\item{payoff}{\itemize{
\item in c("call", "put", "cash_or_nothing_call",
"cash_or_nothing_put", "asset_or_nothing_call", "asset_or_nothing_put")
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "vega",
"theta", "rho", "epsilon", "lambda", "gamma", "vanna", "charm", "vomma",
"veta", "vera", "speed", "zomma", "color", "ultima")
}}

\item{antithetic}{\itemize{
\item if TRUE, antithetic random numbers will be chosen to
decrease variance
}}

\item{...}{\itemize{
\item ...    Other arguments passed on to methods
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
Greeks are derivatives of the option value with respect to
underlying parameters.
For instance, the Greek
\eqn{\Delta = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}
(Delta) measures how the price of an option changes with a minor change in
the underlying asset's price, while
\eqn{\Gamma = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}
(Gamma) measures how \eqn{\Delta} itself changes as the price of the
underlying asset shifts.
Greeks can be computed for different types of options:
For
\itemize{
\item \strong{European Greeks} see also \link{BS_European_Greeks} and \link{Malliavin_European_Greeks}
\item \strong{American Greeks} see also \link{Binomial_American_Greeks}
\item \strong{Asian Greeks} see also \link{BS_Malliavin_Asian_Greeks} and \link{Malliavin_Asian_Greeks}
\item \strong{Geometric Asian Greeks} see also \link{BS_Geometric_Asian_Greeks} and \link{Malliavin_Asian_Greeks}
}

The Greeks are defined as the following partial derivatives of the option
value:
\itemize{
\item \code{Delta} = \eqn{\Delta = \frac{\partial \text{fair\_value}}{\partial \text{initial\_price}}}, the derivative with respect to the price of the
underlying asset
\item \code{Vega} = \eqn{\mathcal{V} = \frac{\partial \text{fair\_value}}{\partial \text{volatility}}},
the derivative with respect to the volatility
\item \code{Theta} = \eqn{\Theta = -\frac{\partial \text{fair\_value}}{\partial \text{time\_to\_maturity}}},
the negative derivative with respect to the time until expiration of the
option
\item \code{rho} = \eqn{\rho = \frac{\partial \text{fair\_value}}{\partial r}},
the derivative with respect to the risk-free interest rate
\item \code{Epsilon} = \eqn{\epsilon = \frac{\partial \text{fair\_value}}{\partial \text{time\_to\_maturity}}},
the derivative with respect to the dividend yield of the underlying asset
\item \code{Lambda} = \eqn{\lambda = \Delta \times \frac{\text{initial\_price}}{\text{exercise\_price}}}
\item \code{Gamma} = \eqn{\Gamma = \frac{\partial^2 \text{fair\_value}}{\partial \text{initial\_price}^2}}, the second derivative with respect to the price of
the underlying asset
\item \code{Vanna} = \eqn{\frac{\partial \Delta}{\partial \text{volatility}} = \frac{\partial^2 \text{fair\_value}}{\partial \text{intial\_price} \, \partial \text{volatility}}}, the derivative of \eqn{\Delta} with respect to the volatility
\item \code{Vomma} = \eqn{\frac{\partial^2 \text{fair\_value}}{\partial \text{volatility}^2}}, the second derivative with respect to the volatility
\item \code{Veta} = \eqn{
\frac{\partial \mathcal V}{\partial r}
= \frac{\partial^2 \text{fair\_value}}{\partial \text{volatility} \, \partial \text{time\_to\_maturity}}},
the derivative of \eqn{\mathcal V} with respect to the time until expiration
of the option
\item \code{Vera} = \eqn{\frac{\partial^2 \text{fair\_value}}{\partial \text{volatiliy} \, \partial \text{r}}},
the derivative of \eqn{\mathcal V} with respect to the risk-free interest rate
\item \code{Speed} = \eqn{\frac{\partial \Gamma}{\partial \text{initial\_price}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{initial\_price}^3}},
the third derivative of the option value with respect to the price of the
underlying asset
\item \code{Zomma} = \eqn{\frac{\Gamma}{\text{volatility}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{volatility}^3}},
the derivative of Gamma with respect to the volatility
\item \code{Color} = \eqn{\frac{\partial \Gamma}{\partial \text{r}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{initial\_price}^2 \partial \text{r}}},
the derivative of Gamma with respect to the risk-free interest rate
\item \code{Ultima} = \eqn{\frac{\partial \text{Vomma}}{\partial \text{volatility}} = \frac{\partial^3 \text{fair\_value}}{\partial \text{volatility}^3}},
the third derivative with respect to the volatility
}

\link{Greeks} computes Greeks for the following option types:
\itemize{
\item \strong{European put- and call options}, which give to option holder the right
but not the obligation to sell (resp. buy) the underlying asset for a
specific price at a specific date.
If $K$ is the exercise price, and \eqn{S_T} the value of the underlying asset
at time-to-maturity \eqn{T}, a European options pay off the following amount
at expiration:
\itemize{
\item \eqn{\max\{K - S_T, 0\}} for a \strong{put-option}
\item \eqn{\max\{S_T - K, 0\}} for a \strong{call-option}
}
\item \strong{American put- and call options} are like European options, but allow
the holder to exercise at any time until expiration
\item \strong{European cash-or-nothing put- and call options} provide the holder with
a fixed amount of cash, if the value of the underlying asset is below (resp.
above) a certain strike price
\item \strong{European asset-or-nothing put- and call options} are similar to
cash-or-nothing options, but provide the holder with one share of the asset.
\item \strong{Asian put- and call options} have a similar payoff to European put- and
call options but differ from European options in that they are path dependent.
Not the price \eqn{S_T} of the underlying asset at time-to-maturity \eqn{T}
is evaluated, but the arithmetic average
\eqn{\frac{1}{T} \int_0^T S_t dt}.
We get the payoffs
\itemize{
\item \eqn{\max\{K - \frac{1}{T} \int_0^T S_t dt, 0\}} for an Asian
\strong{put-option}
\item \eqn{\max\{\frac{1}{T} \int_0^T S_t dt - K, 0\}} for an Asian
\strong{call-option}
}
\item \strong{Geometric Asian options} differ from Asian options in that the geometric
average
\eqn{\exp \left( \frac{1}{T} \int_0^T \ln S_t dt \right)} is evaluated.
}

For reference see Hull (2022) or

\href{https://en.wikipedia.org/wiki/Greeks_(finance)}{en.wikipedia.org/wiki/Greeks_(finance)}.
}
\examples{
Greeks(initial_price = 100, exercise_price = 120, r = 0.01,
time_to_maturity = 5, volatility = 0.30, payoff = "call")

Greeks(initial_price = 100, exercise_price = 100, r = -0.005,
time_to_maturity = 1, volatility = 0.30, payoff = "put",
option_type = "American")

}
\references{
Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition). Pearson

\href{https://en.wikipedia.org/wiki/Greeks_(finance)}{en.wikipedia.org/wiki/Greeks_(finance)}
}
\seealso{
\link{BS_European_Greeks} for option_type = "European"

\link{Binomial_American_Greeks} for option_type = "American"

\link{BS_Geometric_Asian_Greeks} for option_type = = "Geometric Asian"
and model = "black_scholes"

\link{BS_Malliavin_Asian_Greeks} for option_type = = "Asian"
and model = "black_scholes" and greek in c("fair_value", "delta", "rho",
"vega")

\link{Malliavin_Asian_Greeks} for more general cases of Asian Greeks

\link{Greeks_UI} for an interactive visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BS_Geometric_Asian_Greeks.R
\name{BS_Geometric_Asian_Greeks}
\alias{BS_Geometric_Asian_Greeks}
\title{Computes the Greeks of a Geometric Asian Option with classical Call- and
Put-Payoff in the Black Scholes model}
\usage{
BS_Geometric_Asian_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  dividend_yield = 0,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma", "vomma")
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset, can also be a
vector
}}

\item{exercise_price}{\itemize{
\item strike price of the option
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{payoff}{\itemize{
\item the payoff function, either a string in ("call", "put")
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "rho",
"vega", "theta", "gamma", "vomma")
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
For the definition of geometric Asian options see
\link{Malliavin_Geometric_Asian_Greeks}.
\link{BS_Geometric_Asian_Greeks} offers a fast and exaction computation of
Geometric Asian Greeks.
}
\examples{
BS_Geometric_Asian_Greeks(initial_price = 110, exercise_price = 100,
r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma"),
payoff = "put")

}
\seealso{
\link{Malliavin_Geometric_Asian_Greeks} for the Monte Carlo
implementation which provides digital and custom payoff functions and also
works for the jump diffusion model

\link{Greeks_UI} for an interactive visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Malliavin_European_Greeks.R
\name{Malliavin_European_Greeks}
\alias{Malliavin_European_Greeks}
\title{Computes the Greeks of a European option with the Malliavin Monte Carlo
Method in the Black Scholes model}
\usage{
Malliavin_European_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  payoff = "call",
  greek = c("fair_value", "delta", "vega", "theta", "rho", "gamma"),
  model = "Black Scholes",
  paths = 10000,
  seed = 1,
  antithetic = FALSE
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset
}}

\item{exercise_price}{\itemize{
\item strike price of the option
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{payoff}{\itemize{
\item the payoff function, either a string in ("call", "put",
"cash_or_nothing_call", "cash_or_nothing_call", "asset_or_nothing_call",
"asset_or_nothing_put"), or a function
}}

\item{greek}{\itemize{
\item Greeks to be calculated in ("fair_value", "delta", "vega",
"theta", "rho", "gamma")
}}

\item{model}{\itemize{
\item the model to be chosen
}}

\item{paths}{\itemize{
\item the number of simulated paths
}}

\item{seed}{\itemize{
\item the seed of the random number generator
}}

\item{antithetic}{\itemize{
\item if TRUE, antithetic random numbers will be chosen to
decrease variance
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}
}
\description{
For details on the definition of Greeks see \link{Greeks}.
For a description of Malliavin Monte Carlo Methods for Greeks see for example
(Hudde & Rüschendorf, 2023).
}
\examples{
Malliavin_European_Greeks(initial_price = 110,
exercise_price = 100, r = 0.02, time_to_maturity = 4.5,
volatility = 0.22, greek = c("fair_value", "delta", "rho"), payoff = "put")

}
\references{
Hudde, A., & Rüschendorf, L. (2023). European and Asian Greeks for Exponential Lévy Processes. Methodol Comput Appl Probab, 25 (39). \doi{10.1007/s11009-023-10014-5}
}
\seealso{
\link{BS_European_Greeks} for the exact and fast implementation for
call-, put- and digital payoff functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BS_Malliavin_Asian_Greeks.R
\name{BS_Malliavin_Asian_Greeks}
\alias{BS_Malliavin_Asian_Greeks}
\title{Computes the Greeks of an Asian option with the Malliavin Monte Carlo Method
in the Black Scholes model}
\usage{
BS_Malliavin_Asian_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  dividend_yield = 0,
  payoff = "call",
  greek = c("fair_value", "delta", "vega", "rho"),
  steps = round(time_to_maturity * 252),
  paths = 1000,
  seed = 1
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset, can also be a
vector
}}

\item{exercise_price}{\itemize{
\item strike price of the option, can also be a vector
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{payoff}{\itemize{
\item the payoff function, either a string in ("call", "put"), or a
function
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "rho",
"vega")
}}

\item{steps}{\itemize{
\item the number of integration steps
}}

\item{paths}{\itemize{
\item the number of simulated paths
}}

\item{seed}{\itemize{
\item the seed of the random number generator
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
For a description of Asian Greeks see \link{Malliavin_Asian_Greeks}.
BS_Malliavin_Asian_Greeks offers a fast implementation in the Black Scholes
model.
}
\examples{
BS_Malliavin_Asian_Greeks(initial_price = 110, exercise_price = 100,
r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
greek = c("fair_value", "delta", "rho"), payoff = "put")

}
\seealso{
\link{Malliavin_Asian_Greeks} for a greater set of Greeks and also
in the jump diffusion model

\link{Greeks_UI} for an interactive visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BS_European_Greeks.R
\name{BS_European_Greeks}
\alias{BS_European_Greeks}
\title{Computes the Greeks of a European call- or put-option, or of digital
options in the Black Scholes model.}
\usage{
BS_European_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  dividend_yield = 0,
  payoff = "call",
  greek = c("fair_value", "delta", "vega", "theta", "rho", "epsilon", "lambda", "gamma",
    "vanna", "charm", "vomma", "veta", "speed")
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset
}}

\item{exercise_price}{\itemize{
\item strike price of the option
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{payoff}{\itemize{
\item in c("call", "put", "cash_or_nothing_call",
"cash_or_nothing_put", "asset_or_nothing_call", "asset_or_nothing_put")
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "vega",
"theta", "rho", "epsilon", "lambda", "gamma", "vanna", "charm", "vomma",
"veta", "vera", "speed", "zomma", "color", "ultima")
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
For details on the definition of Greeks see \link{Greeks}.
}
\examples{
BS_European_Greeks(initial_price = 120, exercise_price = 100,
r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
greek = c("fair_value", "delta", "gamma"), payoff = "put")
}
\seealso{
\link{Malliavin_European_Greeks} for the Monte Carlo implementation

\link{Greeks_UI} for an interactive visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BS_Implied_Volatility.R
\name{BS_Implied_Volatility}
\alias{BS_Implied_Volatility}
\title{Computes the implied volatility for European put- and call options in the
Black Scholes model via Halley's method.}
\usage{
BS_Implied_Volatility(
  option_price,
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  dividend_yield = 0,
  payoff = "call",
  start_volatility = 0.3,
  precision = 1e-09
)
}
\arguments{
\item{option_price}{\itemize{
\item current price of the option
}}

\item{initial_price}{\itemize{
\item initial price of the underlying asset.
}}

\item{exercise_price}{\itemize{
\item strike price of the option.
}}

\item{r}{\itemize{
\item risk-free interest rate.
}}

\item{time_to_maturity}{\itemize{
\item time to maturity.
}}

\item{dividend_yield}{\itemize{
\item dividend yield.
}}

\item{payoff}{\itemize{
\item the payoff function, a string in ("call", "put").
}}

\item{start_volatility}{\itemize{
\item the volatility value to start the approximation
}}

\item{precision}{\itemize{
\item precision of the result
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
For the definition of \emph{implied volatility} see
\link{Implied_Volatility}.
\link{BS_Implied_Volatility} offers a very fast implementation for European put-
and call options applying Halley's method (see

\href{https://en.wikipedia.org/wiki/Halley\%27s_method}{en.wikipedia.org/wiki/Halley\%27s_method}).
}
\examples{
BS_Implied_Volatility(option_price = 27, initial_price = 100,
exercise_price = 100, r = 0.03, time_to_maturity = 5, dividend_yield = 0.015,
payoff = "call")
}
\seealso{
\link{Implied_Volatility} for American and Asian options, and for
digital payoff functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Binomial_American_Greeks.R
\name{Binomial_American_Greeks}
\alias{Binomial_American_Greeks}
\title{Computes the Greeks of an American call- or put-option with the Binomial
options pricing model}
\usage{
Binomial_American_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  dividend_yield = 0,
  payoff = "call",
  greek = c("fair_value", "delta", "vega", "theta", "rho", "epsilon", "gamma"),
  steps = 1000,
  eps = 1/1e+05
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset.
}}

\item{exercise_price}{\itemize{
\item strike price of the option.
}}

\item{r}{\itemize{
\item risk-free interest rate.
}}

\item{time_to_maturity}{\itemize{
\item time to maturity.
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset.
}}

\item{dividend_yield}{\itemize{
\item dividend yield.
}}

\item{payoff}{\itemize{
\item the payoff function, a string in ("call", "put").
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "vega",
"theta", "rho", "epsilon", "gamma")
}}

\item{steps}{\itemize{
\item the number of integration steps.
}}

\item{eps}{\itemize{
\item the step size for the finite difference method to calculate
theta, vega, rho and epsilon
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
In contract to European Options, American options can be executed at any time
until the expiration date.
For more details on the definition of Greeks in general see \link{Greeks}.
This functions computes Greeks of American put- and call options in the
binomial option pricing model (see (Hull, 2022)).
}
\examples{
Binomial_American_Greeks(initial_price = 100, exercise_price = 100,
r = 0, time_to_maturity = 1, volatility = 0.3, dividend_yield = 0,
payoff = "call", greek = c("fair_value", "delta", "vega", "theta", "rho",
"epsilon", "gamma"), steps = 20)

}
\references{
Hull, J. C. (2022). Options, futures, and other derivatives (11th Edition). Pearson
}
\seealso{
\link{Greeks_UI} for an interactive visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Greeks_UI.R
\name{Greeks_UI}
\alias{Greeks_UI}
\title{Opens a shiny app to interactively visualize option prices and Greeks.}
\usage{
Greeks_UI()
}
\description{
Opens a shiny app to interactively visualize option prices and Greeks.
This works for European Options (see \link{BS_European_Greeks}), American
Options (see \link{Binomial_American_Greeks}), Geometric Asian Options (see
\link{BS_Geometric_Asian_Greeks}), as well as Asian options (see
\link{BS_Malliavin_Asian_Greeks}).
For performance reasons, just the Black-Scholes model is possible, and for
some cases, the set of Greeks is limited.
On the y-Axis, the option value resp. the value of the greeks are displayed,
for the x-axis, several parameters like \code{initial_price} or \code{time_to_maturity}
are possible.

\if{latex}{
\figure{GreeksUI.png}{The interface of the functions}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Malliavin_Geometric_Asian_Greeks.R
\name{Malliavin_Geometric_Asian_Greeks}
\alias{Malliavin_Geometric_Asian_Greeks}
\title{Computes the Greeks of a geometric Asian option with the Malliavin Monte
Carlo Method in the Black Scholes- or Jump diffusion model}
\usage{
Malliavin_Geometric_Asian_Greeks(
  initial_price = 100,
  exercise_price = 100,
  r = 0,
  time_to_maturity = 1,
  volatility = 0.3,
  dividend_yield = 0,
  payoff = "call",
  greek = c("fair_value", "delta", "rho", "vega", "theta", "gamma"),
  model = "black_scholes",
  lambda = 0.2,
  alpha = 0.3,
  jump_distribution = function(n) stats::rt(n, df = 3),
  steps = round(time_to_maturity * 252),
  paths = 10000,
  seed = 1,
  antithetic = FALSE
)
}
\arguments{
\item{initial_price}{\itemize{
\item initial price of the underlying asset, can also be a
vector
}}

\item{exercise_price}{\itemize{
\item strike price of the option, can also be a vector
}}

\item{r}{\itemize{
\item risk-free interest rate
}}

\item{time_to_maturity}{\itemize{
\item time to maturity in years
}}

\item{volatility}{\itemize{
\item volatility of the underlying asset
}}

\item{dividend_yield}{\itemize{
\item dividend yield
}}

\item{payoff}{\itemize{
\item the payoff function, either a string in ("call", "put",
"digital_call", "digital_put"), or a function
}}

\item{greek}{\itemize{
\item Greeks to be calculated in c("fair_value", "delta", "rho", "vega", "theta", "gamma)
}}

\item{model}{\itemize{
\item the model to be chosen in ("black_scholes", "jump_diffusion")
}}

\item{lambda}{\itemize{
\item the lambda of the Poisson process in the jump-diffusion model
}}

\item{alpha}{\itemize{
\item the alpha in the jump-diffusion model influences the jump size
}}

\item{jump_distribution}{\itemize{
\item the distribution of the jumps, choose a function
which generates random numbers with the desired distribution
}}

\item{steps}{\itemize{
\item the number of integration steps
}}

\item{paths}{\itemize{
\item the number of simulated paths
}}

\item{seed}{\itemize{
\item the seed of the random number generator
}}

\item{antithetic}{\itemize{
\item if TRUE, antithetic random numbers will be chosen to
decrease variance
}}
}
\value{
Named vector containing the values of the Greeks specified in the
parameter \code{greek}.
}
\description{
In contrast to Asian options (see \link{Malliavin_Asian_Greeks}), geometric Asian
options evaluate the geometric average
\eqn{\exp \left( \frac{1}{T} \int_0^T \ln S_t dt \right)}, where
\eqn{S_t} is the price of the underlying asset at time \eqn{t} and \eqn{T} is
the time-to-maturity of the option (see

\href{https://en.wikipedia.org/wiki/Asian_option#European_Asian_call_and_put_options_with_geometric_averaging}{en.wikipedia.org/wiki/Asian_option#European_Asian_call_and_put_options_with_geometric_averaging}).
For more details on the definition of Greeks see \link{Greeks}, and for a
description of the Malliavin Monte Carlo Method for Greeks see for example
(Hudde & Rüschendorf, 2023).
}
\examples{
Malliavin_Asian_Greeks(initial_price = 110, exercise_price = 100,
r = 0.02, time_to_maturity = 4.5, dividend_yield = 0.015, volatility = 0.22,
greek = c("fair_value", "delta", "rho"), payoff = "put")

}
\references{
Hudde, A., & Rüschendorf, L. (2023). European and Asian Greeks for Exponential Lévy Processes. Methodol Comput Appl Probab, 25 (39). \doi{10.1007/s11009-023-10014-5}
}
\seealso{
\link{BS_Geometric_Asian_Greeks} for exact and fast computation in the
Black Scholes model and for put- and call payoff functions
}
